<div class="b-settings-folders g-ui-user-select-none row" data-bind="css: { 'ignore-folder-subscribe': !useImapSubscribe }">
	<div class="span8 width100-on-mobile">
		<div class="form-horizontal">
			<div class="legend">
				<span class="i18n" data-i18n="SETTINGS_FOLDERS/LEGEND_FOLDERS"></span>
				&nbsp;&nbsp;&nbsp;
				<i class="icon-spinner animated" style="margin-top: 5px" data-bind="visible: loading"></i>
			</div>
		</div>
		<a class="btn" data-bind="click: createFolder">
			<i class="icon-folder-add"></i>
			&nbsp;&nbsp;
			<span class="i18n" data-i18n="SETTINGS_FOLDERS/BUTTON_CREATE"></span>
		</a>
		&nbsp;&nbsp;
		<a class="btn" data-bind="click: systemFolder">
			<i class="icon-wrench"></i>
			&nbsp;&nbsp;
			<span class="i18n" data-i18n="SETTINGS_FOLDERS/BUTTON_SYSTEM"></span>
		</a>
		<div class="alert folders-list-error" data-bind="visible: folderList.optimized">
			<b class="i18n" data-i18n="SETTINGS_FOLDERS/TO_MANY_FOLDERS_DESC_1"></b>
			<br />
			<span class="i18n" data-i18n="SETTINGS_FOLDERS/TO_MANY_FOLDERS_DESC_2"></span>
		</div>
		<div class="alert folders-list-error" data-bind="visible: '' !== folderList.error()">
			<button type="button" class="close" data-bind="click: function () { folderList.error(''); }">&times;</button>
			<span data-bind="text: folderList.error"></span>
		</div>
		<table class="table table-hover list-table" data-bind="i18nUpdate: folderList" style="margin-top: 40px">
			<colgroup>
				<col />
				<col style="width: 1%" />
				<col style="width: 1%" />
				<col style="width: 1%" data-bind="visible: displaySpecSetting" />
			</colgroup>
			<tbody data-bind="template: { name: 'SettingsFolderItem', foreach: folderList }"></tbody>
		</table>
	</div>
	<div class="span4 hide-on-mobile" style="position: relative" data-bind="visible: '' !== folderListHelp()">
		<div class="alert alert-info" style="position: fixed">
			<span data-bind="text: folderListHelp"></span>
		</div>
	</div>
</div>