
import {componentExportHelper} from 'Component/Abstract';
import {AbstractInput} from 'Component/AbstractInput';

class InputComponent extends AbstractInput {}

export default componentExportHelper(InputComponent, 'InputComponent');
