<!DOCTYPE html>
<html lang="en" dir="ltr">
	<head>
		<meta charset="utf-8" />
		<meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1" />
		<meta name="robots" content="noindex, nofollow, noodp" />
		<meta name="google" content="notranslate" />
		<title>{{ErrorTitle}}</title>
		<style type="text/css">

			html, body {
				font-family: arial, sans-serif;
				font-size: 14px;
			}

			.content {
				position: absolute;
				left: 50%;
				top: 50%;
				text-align: center;
				width: 600px;
				margin: -200px 0px 0px -250px;
				background: #FFF;
				line-height: 150%;
				border-bottom: 1px solid #E4E4E4;
			}

			.error-header {
				font-size: 16px;
				font-weight: bold;
			}

			.error-desc {
				padding: 10px;
				background-color: #eee;
			}

		</style>
	</head>
	<body class="content">
		<div class="error-header">
			{{ErrorHeader}}
		</div>
		<br />
		<div class="error-desc">
			<br />
			{{ErrorDesc}}
			<br />
			<br />
			<div style="{{BackLinkVisibilityStyle}}">
				<br />
				<a href="{{BackHref}}">{{BackLink}}</a>
			</div>
		</div>
	</body>
</html>