
.b-admin-about {
	.rl-logo {
		display: inline-block;
		width: 250px;
		height: 250px;
		margin-top: -10px;
		margin-bottom: -10px;
		background-image: url("images/rainloop-logo.png");
	}
	.rl-desc {
		margin-top: 20px;
		margin-left: -20px;
	}
}

