
.b-compose {

	&.modal {

		width: 850px;
		margin: 10px auto;

		.modal-body {
			overflow: auto;
			padding: 0px;
		}
	}

	.textAreaParent, .attachmentAreaParent {
		overflow: hidden;
		position: relative;
	}

	.attachmentAreaParent {
		padding: 10px 10px 6px 10px;
		background: #ddd;
		border-top: 1px solid #ccc;

		overflow-y: auto;
	}

	.attachmentAreaParent {

		.no-attachments-desc {
			padding: 50px 10px;
			text-align: center;
			font-size: 24px;
			color: #666;
			text-shadow: 0 1px 0 #fff;
			text-shadow: 0 1px 0 rgba(255, 255, 255, 0.5);
		}

		.attachmentList {
			margin: 0;
			padding: 10px;
		}
	}

	.b-header-toolbar {

		height: 40px;

		color: #fff;
		background-color: @rlMainDarkColor;

		background-color: #333;
		background-color: rgba(0,0,0,0.8) !important;

		.close, .close-custom, .minimize-custom {
			opacity: 1;
			color: #fff;
			border-color: #eee;

			font-size: 24px;
			line-height: 24px;
		}

		.btn.disabled {
			&.button-delete {
				visibility: hidden;
			}
		}

		.button-save, .button-delete, .saved-text {
			margin-left: 8px;
		}

		.button-close, .button-skip {
			margin-left: 8px;
		}

		.disabled.button-delete {
			margin-left: 0px;
		}
	}

	.b-header {

		padding: 10px;
		background-color: #eee;
		color: #333;

		html.rl-mobile &{
			overflow: scroll;
		}

		.e-identity {

			color: #333;
			text-decoration: none;
			font-weight: bold;

			&:hover {
				color: #333;
				text-decoration: none;
			}

			&.multiply {
				cursor: pointer;
				border-bottom: 1px dashed #555;
			}
		}

		.e-row {
			line-height: 30px;
		}

		.e-label {
			text-align: right;
			width: 1%;
			min-width: 70px;
			padding: 6px 10px;

			html.rl-mobile &{
				min-width: 50px;
				padding: 6px 10px 6px 0px;
			}
		}

		.e-value {

			padding: 2px 0px;

			textarea, input[type="text"] {
				width: 98%;
			}
			textarea {
				height: 40px;
			}
		}

		.error-desc {
			color: red;
		}

		.error-to {
			color: red;
			font-weight: bold;
		}

		.b-appachments {

			.b-attacment {
				line-height: 20px;
				padding-bottom: 10px;
			}

			.b-attacment-in-process {

				line-height: 20px;
				padding-bottom: 10px;

				.uploading {
					display: none;
					padding-right: 5px;
				}

				&.uploading .uploading {
					display: inline;
				}

				.upload-progress {
					font-weight: bold;
				}

				&.error .namedStr {
					color: #888;
				}
				.error {
					color: red;
				}
				.close, .close-custom {
					float: left;
					padding-right: 13px;
				}
			}
		}
	}

	.cke_chrome {
		border-left-width: 0 !important;
		border-right-width: 0 !important;
		border-bottom-width: 0 !important;
	}

	.b-attachment-button {
		display: inline-block;
	}

	.b-attachment-place {

		position: absolute;
		left: 5px;
		right: 5px;
		top: 5px;
		bottom: 5px;

		border: 2px #777 dashed;
		z-index: 300;

		line-height: 119px;
		text-align: center;
		background-color: #efefef;
		font-size: 24px;

		border-radius: 10px;

		&.dragAndDropOver {
			background-color: #fff;
		}
	}
}
