html.rl-no-preview-pane {

	.messageView {
		display: none;

		&.message-selected {
			display: block;
		}
	}
}

.messageView {

	z-index: 100;

	.toolbar {
		position: absolute;
		top: 0;
		right: 0;
		left: 0;
		height: 30px;
		padding: 10px 0;
		color: #fff;
	}

	.b-content {

		position: absolute;
		margin: 0;
		top: 50px + @rlLowMargin;
		bottom: @rlLowMargin + @rlBottomMargin;
		right: @rlLowMargin;
		left: -2px;
		/*overflow: hidden;*/
		border: @rlLowBorderSize solid @rlMainDarkColor;

		.border-radius(@rlLowBorderRadius);

		background-color: #fff;

		.b-message-view-checked-helper {
			text-align: center;
			font-size: 70px;
			line-height: 70px;
			padding-top: 100px;
			color: #999;

			.icon-mail {
				font-size: 100px;
				font-size: 50px;
				line-height: 90px;
				padding-left: 10px;
			}
		}

		.logoPlace {
			text-align: center;
		}

		.b-message-view-desc {
			text-align: center;
			font-size: 24px;
			line-height: 30px;
			color: #999;
			padding: 120px 10px 0 10px;
		}

		.b-message-view-desc.error {
			color: #DA4F49;
		}

		.b-message-view-iframe {
		}

		.b-message-view-iframe-backdrop {
			display: none;
			background: transparent;
			position: absolute;
			width: 100%;
			height: 100%;
			top: 0px;
			left: 0px;
			right: 0px;
			bottom: 0px;
			opacity: .1;
		}

		.content {
			-webkit-overflow-scrolling: touch;
		}

		.message-fixed-button-toolbar {
			z-index: 100;
			position: absolute;
			top: 33px;
			right: 10px;
		}

		.infoParent {
			cursor: pointer;
			margin: 0 2px 0 5px;
			opacity: 0.3;

			&:hover {
				opacity: 0.6;
			}
		}

		.flagParent {

			cursor: pointer;
			margin: 0 20px 0 2px;

			.flagOn {
				color: orange;
			}

			.flagOff {
				opacity: 0.5;
				&:hover {
					opacity: 1;
				}
			}
		}

		.messageItemHeader {

			position: absolute;
			padding: 10px;
			background-color: #f8f8f8;
			border-top: 0px;
			border-bottom: 1px solid #ddd;
			border-top-right-radius: 5px;
			border-top-left-radius: 5px;
			z-index: 1;
			top: 0;
			left: 0;
			right: 0;

			.date {
			}

			.fromPic {
				display: inline-block;
				width: 50px;
				height: 50px;
				float: left;
				padding: 2px;
				margin: 0 5px 0 0;
				background: #fff;
				border: 1px solid #ccc;
				border-radius: 6px;
				box-sizing: border-box;
			}

			.subjectParent {
				font-size: 18px;
				font-weight: bold;
				text-overflow: ellipsis;
				overflow: hidden;
				white-space: nowrap;
				margin-bottom: 8px;
				line-height: 100%;
				height: 22px;
				vertical-align: middle;
			}

			.messageButtons {
				margin-top: 5px;
			}

			.informationShort {
				margin: 4px 0;
				margin-left: 5px;
				margin-right: 50px;

				a {
					.g-ui-link;
				}

				html.rl-mobile &{
					text-overflow: ellipsis;
					overflow: hidden;
					white-space: nowrap;
				}
			}

			.informationShortWrp {
				max-height: 100px;
				overflow-y: auto;
				min-height: 30px;
				margin-top: 4px;
			}

			.informationFull {
				margin-top: 10px;
				border: 1px solid #ddd;
				background-color: #fff;
				border-radius: 5px;
				padding: 8px 0;
				overflow: hidden;

				.size {
					margin-left: 4px;
				}

				table {
				    width: 100%;
				}

				table, tr, td {
					border-spacing: 0;
				}

				td {
					padding: 0 10px;
					vertical-align: top;
					min-width: 43px;
				}

				td:first-child {
					border-right: 1px solid #ddd;
					text-align: right;
					width: 1%;
				}

				.uiLine {
				}

				.uiLabel {
					white-space: nowrap;
				}

				.uiLabelValue {
					word-break: break-all;
				}
			}

			.emptySubjectText {
				display: none;
				font-style: italic;
				font-weight: normal;
				color: #999;
			}

			&.emptySubject .emptySubjectText {
				display: inline;
			}
		}

		.messageItem {

			position: absolute;
			top: 0px;
			bottom: 0;
			left: 0;
			right: 0;
			overflow: auto;
			-webkit-overflow-scrolling: touch;
			border-radius: @rlLowBorderRadius;

			.buttonUp, .buttonUnFull, .buttonFull {
				display: inline-block;
				position: fixed;
				right: 25px;
				bottom: 25px;
				height: 30px;
				width: 30px;
				text-align: center;
				vertical-align: middle;
				line-height: 30px;
				background-color: transparent;
				background-color: #fff;
				border: 1px solid #333;
				color: #333;
				z-index: 2;
				cursor: pointer;
				border-radius: 5px;
				.opacity(50);

				&:hover {
					.opacity(80);
					border-color: #666;
					background-color: #888;
					color: #fff;
				}
			}

			.buttonUp {
				right: 65px;
				z-index: 0;
			}

			.buttonUp, .buttonUnFull {
				display: none;
			}

			.loading {
				text-align: center;
				font-size: 24px;
				color: grey;
				padding-top: 50px;
			}

			.line-loading {
				height: 0px;
			}

			.showImages, .readReceipt, .pgpSigned, .pgpEncrypted {
				cursor: pointer;
				padding: 10px 15px;
				border-bottom: 1px solid #ddd;
				background-color: #eee;
			}

			.pgpInfo {
				padding: 5px 15px;
				border-bottom: 1px solid #ddd;
				background-color: #fcf8e3;

				&.success {
					background-color: #e9f4ff;
				}
			}

			.readReceipt {
				background-color: #ffffd9;
			}

			.attachmentsPlace {

				padding: 10px 10px 6px 10px;
				background: #eee;
				border-bottom: 1px solid #ddd;
				position: relative;

				.attachmentList {
					margin: 0;
				}

				.checkedParent {
					display: none;
				}

				&.selection-mode {
					.checkedParent {
						display: inline-block;
					}
				}

				&.unselectedAttachmentsError {
					.attachmentItem {
						box-shadow: 0 1px 4px red;
						box-shadow: 0 1px 5px rgba(255, 0, 0, 0.4);
						box-shadow: 0 0 0 1px rgba(255, 0, 0, 0.2), 0 1px 5px rgba(255, 0, 0, 0.3);
					}
				}

				.controls-handle {
					position: absolute;
					bottom: 5px;
					right: 8px;
					color: #999;
					cursor: pointer;
				}
			}

			.attachmentsControls {
				padding: 7px 5px 7px 14px;
				background: #e8e8e8;
				border-bottom: 1px solid #ddd;
			}

			.rlBlockquoteSwitcher {

				background-color: #eee;
				border: 1px solid #999;
				display: inline-block;
				width: 30px;
				height: 14px;
				line-height: 14px;
				text-align: center;
				cursor: pointer;
				margin: 10px 0px;

				opacity: 0.5;
				&:hover {
					opacity: 1;
				}
			}

			.bodySubHeader {
				z-index: 2;
			}

			.bodyText {

				color: #000;
				font-family: Arial, Verdana, Geneva, sans-serif;

				.b-text-part {

					height: 100%;

					div[data-x-div-type=html] {
						height: 100%;
						div[data-x-div-type=body] {
							height: 100%;
						}
					}

					a {
						color: blue;
						text-decoration: underline;

						&:visited {
							color: #609;
						}
						&:active {
							color: red;
						}
					}

					blockquote {
						border-left: 2px solid #000;
						padding: 0px 10px;
						margin: 0;
					}

					.rl-bq-switcher.hidden-bq {
						display: none;
					}

					&.html {

						div[data-x-div-type=body] {
							/*padding: 15px;*/
							margin: 15px;
						}

						pre, code {
							margin: 0px;
							padding: 0px;
							background: #fff;
							border: none;
							border-radius: 0;
							font-family: Monaco, Menlo, Consolas, 'Courier New', monospace;
							display: block;
							word-break: normal;
							word-wrap: break-word;
							background-color: #f9f9f9;
						}

						code {
							display: inline;
							padding: 2px 5px;
						}

						pre {
							padding: 5px 10px;
							border-radius: 5px;
							background-color: #f9f9f9;
						}

						pre > code {
							padding: 0px;
						}
					}

					&.plain {

						padding: 15px;
						/*white-space: pre-wrap;*/
						font-family: Monaco, Menlo, Consolas, 'Courier New', monospace;

						pre {
							margin: 0px;
							padding: 0px;
							background: #fff;
							border: none;
							font-family: Monaco, Menlo, Consolas, 'Courier New', monospace;
							display: block;
							word-break: normal;
						}

						pre.b-plain-openpgp {
							display: inline-block;
							padding: 6px 10px;
							border: 1px dashed #666;
							background: #fff;
							word-break: break-all;

							&.success {
								border-color: green;
								background-color: rgba(0, 255, 0, 0.03);
							}
							&.error {
								border-color: red;
								background-color: rgba(255, 0, 0, 0.03);
							}
						}

						blockquote {
							border-left: 2px solid blue;
							color: blue;
						}

						blockquote blockquote {
							border-left: 2px solid green;
							color: green;
						}

						blockquote blockquote blockquote {
							border-left: 2px solid red;
							color: red;
						}
					}

					.b-openpgp-control {

						display: inline-block;
						cursor: pointer;
						color: #777;
						/*float: left;*/

						&:hover {
							color: #111;
						}

						&.success {
							color: green;
							cursor: help;
						}

						&.error {
							color: red;
						}
					}
				}
			}
		}
	}

	&.message-focused .b-content {
		z-index: 101;
		box-shadow: 0px 2px 8px rgba(0, 0, 0, 0.2);
		border-radius: @rlLowBorderRadius;
		border-color: darken(@rlMainDarkColor, 5%);
	}

	.thread-controls {
		.dropdown-toggle {
			padding-left: 10px;
			padding-right: 10px;
		}
		&.open .dropdown-toggle {
			padding-left: 10px;
			padding-right: 10px;
		}
	}

	.thread-list {

		.e-link {
			padding: 4px 8px 6px 10px;
		}

		&.hide-more .thread-list-message.more-that {
			display: none;
		}

		.thread-date {
			font-size: 13px;
			color: #999;
		}

		.more-threads {
			text-align: center;
			padding: 8px;
			background-color: #F5F5F5;
			color: #555;
			text-decoration: underline;
			border-top: 1px dashed #555;
		}
	}
}

html.rl-resizer .b-message-view-iframe-backdrop {
	display: block !important;
}

html.rl-no-preview-pane .messageView {
	.toolbar {
		padding-left: @rlMainBorderSize;
	}

	.b-content {
		top: 50px;
		left: 0;
		bottom: @rlBottomMargin;
		right: @rlBottomMargin;
		border: @rlMainBorderSize solid @rlMainDarkColor;
		box-shadow: @rlMainShadow;
		border-radius: @rlMainBorderRadius;
	}
}

html.rl-bottom-preview-pane .messageView {
	.b-content {
		bottom: @rlBottomMargin;
	}
}

html.cssanimations .messageItem .line-loading {
	height: 5px !important;
}

html.rl-message-fullscreen {

	#rl-left, #rl-bottom {
		display: none !important;
	}

	#rl-right {
		.RL-MailMessageList, .RL-SettingsPane, .RL-SystemDropDown, .RL-MailMessageView .messageView .toolbar {
			display: none !important;
		}
	}

	.messageView .b-content {
		position: fixed !important;
		margin: 5px !important;
		top: 0 !important;
		left: 0 !important;
		right: 0 !important;
		bottom: 0 !important;
		z-index: 10000 !important;
		border: @rlLowBorderSize solid @rlMainDarkColor  !important;
		border-radius: @rlLowBorderRadius  !important;
	}
}

html .messageItem {
	.buttonUp, .buttonUp {
		display: none !important;
	}
	&.scroller-shadow-top .buttonUp {
		display: inline-block !important;
	}
}

html.rl-desktop .messageItem {
	.buttonUp, .buttonFull {
		display: none !important;
	}
	&:hover {
		&.scroller-shadow-top .buttonUp, .buttonFull {
			display: inline-block !important;
		}
	}
}

html.rl-message-fullscreen .messageItem {
	.buttonUnFull {
		display: inline-block !important;
	}
	.buttonFull {
		display: none !important;
	}
	.buttonUp {
		display: none !important;
	}
	&.scroller-shadow-top .buttonUp {
		display: inline-block !important;
	}
}
