<input class="i18n" type="text" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"
	data-bind="textInput: value, attr: {'placeholder': placeholder}, enable: enable, css: className" />
<!-- ko if: labeled -->
	&nbsp;
	<span class="i18n" data-bind="attr: {'data-i18n': label}"></span>
	&nbsp;
<!-- /ko -->
<!-- ko if: triggered -->
	&nbsp;
	<div data-bind="component: {
		name: 'SaveTrigger',
		params: { value: trigger }
	}"></div>
<!-- /ko -->