@keyframes redirectingRotation {
	to {
		transform: rotate(1turn);
	}
}

html, body {
	padding: 0;
	margin: 0;
	overflow: hidden;
}

.social-icon-wrp {
	position: absolute;
	top: 0;
	left: 0;
	bottom: 0;
	right: 0;
}

.social-icon {
	position: absolute;
	top: 50%;
	left: 50%;
	font-size: 140px;
	margin: -60px 0 0 -60px;
	color: #999;
}

.redirecting:after{
	content: '';
	position: absolute;
	width: 240px;
	height: 240px;
	top: 50%;
	left: 50%;
	margin-top: -120px;
	margin-left: -120px;

	border: 8px solid transparent;
	border-color: transparent;
	border-top-color: #999;
	animation: redirectingRotation 1s infinite ease-in-out;

	border-radius: 50%;
	z-index: 1;
}
