.popups {

	.b-two-factor-content {

		width: 750px;

		.modal-body {
			min-height: 100px;
		}

		.modal-header {
			background-color: #fff;
		}
	}

	.b-two-factor-test-content {
		.modal-header {
			background-color: #fff;
		}
	}
}
