<div class="b-admin-licensing" >
	<div class="row">
		<div class="alert alert-info span8" style="margin-top: 10px;">
			<span data-i18n="[html]TAB_LICENSING/HTML_ALERT_TOP_1"></span>
			&nbsp;
			<a href=http://www.rainloop.net/licensing/ target=_blank>RainLoop Software License</a>.
		</div>
	</div>
	<br />
	<div class="form-horizontal">
		<div class="control-group">
			<label class="control-label" data-i18n="TAB_LICENSING/LABEL_YOUR_DOMAIN"></label>
			<div class="controls">
				<span class="help-inline" style="padding-top: 5px;">
					 <strong data-bind="text: adminDomain"></strong>
				 </span>
			</div>
		</div>
		<div class="control-group" data-bind="visible: !subscriptionEnabled()">
			<label class="control-label" style="padding-top: 14px" data-i18n="TAB_LICENSING/LABEL_VERSION"></label>
			<div class="controls">
				<div class="alert alert-block span6" style="margin-left: 0">
					<h4>Basic</h4>
					<br />
					<p data-i18n="TAB_LICENSING/TYPE_BASIC_HINT_2"></p>
				</div>
			</div>
		</div>
		<div class="control-group" data-bind="visible: subscriptionEnabled">
			<label class="control-label" style="padding-top: 20px" data-i18n="TAB_LICENSING/LABEL_VERSION"></label>
			<div class="controls">
				<div data-bind="visible: licensingProcess()" style="padding-top: 20px">
					<i class="icon-spinner animated"></i>
					&nbsp;&nbsp;
					<span data-i18n="TAB_LICENSING/LABEL_CHECKING"></span>&hellip;
				</div>
				<div data-bind="visible: !licensingProcess()">
					<div class="alert alert-block span6" style="margin-left: 0;  padding-top: 20px" data-bind="visible: !licensing()">
						<h4>Basic</h4>
						<br />
						<p data-i18n="TAB_LICENSING/TYPE_BASIC_HINT"></p>
					</div>
					<div data-bind="visible: licensing()">
						<div class="alert alert-success span6" style="margin-left: 0; margin-top: 10px; padding-top: 20px" data-bind="visible: licenseValid() && '' === licenseError()">
							<h4>
								<snan>Premium</snan>
								<span data-bind="visible: licenseIsUnlim()">
									&nbsp;
									(<snan data-i18n="TAB_LICENSING/TYPE_PREMIUM_LIFETIME"></snan>)
								</span>
							</h4>
							<p data-bind="visible: 0 < licenseExpired()" style="padding-top: 10px">
								<b data-i18n="TAB_LICENSING/LABEL_SUB_EXPIRES"></b>:
								&nbsp;
								<span data-bind="text: licenseExpiredMomentValue()"></span>
							</p>
						</div>
						<div class="alert alert-error span6" style="margin-left: 0; margin-top: 10px;" data-bind="visible: !licenseValid() && '' !== licenseError(), text: licenseError"></div>
					</div>
				</div>
			</div>
		</div>
		<div class="control-group" data-bind="visible: subscriptionEnabled() && !licensingProcess()">
			<div class="controls">
				<a class="btn" data-bind="click: showActivationForm">
					<i class="icon-happy-smiley"></i>
					&nbsp;&nbsp;
					<span data-i18n="TAB_LICENSING/BUTTON_ACTIVATE"></span>
				</a>
				&nbsp;&nbsp;
				<a class="btn" href="http://www.rainloop.net/purchase/" target="_black">
					<i class="icon-purchase"></i>
					&nbsp;&nbsp;
					<span data-i18n="TAB_LICENSING/BUTTON_PURCHASE"></span>
				</a>
<!--
				&nbsp;&nbsp;
				<a class="btn" data-bind="click: showTrialForm, visible: !licensing()">
					<i class="icon-eye"></i>
					&nbsp;&nbsp;
					<span data-i18n="TAB_LICENSING/BUTTON_TRIAL"></span>
				</a>
-->
			</div>
		</div>
	</div>
</div>