<div class="b-admin-branding">
	<div class="form-horizontal">
		<div class="legend" data-i18n="TAB_BRANDING/LEGEND_BRANDING"></div>
		<div class="control-group">
			<label class="control-label" data-i18n="TAB_BRANDING/LABEL_PAGE_TITLE"></label>
			<div class="controls">
				<div data-bind="component: {
					name: 'Input',
					params: {
						value: title,
						trigger: title.trigger,
						size: 5
					}
				}"></div>
			</div>
		</div>
		<div class="control-group">
			<label class="control-label" data-i18n="TAB_BRANDING/LABEL_LOADING_DESCRIPTION"></label>
			<div class="controls">
				<div data-bind="component: {
					name: 'Input',
					params: {
						value: loadingDesc,
						trigger: loadingDesc.trigger,
						size: 5
					}
				}"></div>
			</div>
		</div>
		<div class="control-group">
			<label class="control-label" data-i18n="TAB_BRANDING/LABEL_FAVICON_URL"></label>
			<div class="controls">
				<div data-bind="component: {
					name: 'Input',
					params: {
						value: faviconUrl,
						trigger: faviconUrl.trigger,
						placeholder: 'https://',
						size: 5
					}
				}"></div>
			</div>
		</div>
	</div>
	<br />
	<ul class="nav nav-tabs" data-bind="visible: !community">
		<li class="active">
			<a class="i18n" data-i18n="TAB_BRANDING/LEGEND_LOGIN"
			   href="#branding-login-section-id" data-toggle="tab"></a>
		</li>
		<li>
			<a class="i18n" data-i18n="TAB_BRANDING/LEGEND_USER"
			   href="#branding-user-section-id" data-toggle="tab"></a>
		</li>
		<li>
			<a class="i18n" data-i18n="TAB_BRANDING/LEGEND_WELCOME_PAGE"
			   href="#branding-welcome-page-section-id" data-toggle="tab"></a>
		</li>
	</ul>
	<br  />
	<div class="row" data-bind="visible: !community">
		<div class="tab-content span12">
			<div class="tab-pane active" id="branding-login-section-id">

<div class="form-horizontal" data-bind="css: {'disabled-form': !capa()}">
	<div class="control-group">
		<label class="control-label" data-i18n="TAB_BRANDING/LABEL_LOGIN_LOGO"></label>
		<div class="controls">
			<div data-bind="component: {
				name: 'Input',
				params: {
					value: loginLogo,
					trigger: loginLogo.trigger,
					placeholder: 'https://',
					size: 5,
					enable: capa
				}
			}"></div>
		</div>
	</div>
	<div class="control-group">
		<label class="control-label" data-i18n="TAB_BRANDING/LABEL_LOGIN_DESCRIPTION"></label>
		<div class="controls">
			<div data-bind="component: {
				name: 'Input',
				params: {
					value: loginDescription,
					trigger: loginDescription.trigger,
					size: 5,
					enable: capa
				}
			}"></div>
		</div>
	</div>
	<div class="control-group">
		<label class="control-label" data-i18n="TAB_BRANDING/LABEL_LOGIN_BACKGROUND"></label>
		<div class="controls">
			<div data-bind="component: {
				name: 'Input',
				params: {
					value: loginBackground,
					trigger: loginBackground.trigger,
					placeholder: 'https://',
					size: 5,
					enable: capa
				}
			}"></div>
		</div>
	</div>
	<div class="control-group">
		<label class="control-label" data-i18n="TAB_BRANDING/LABEL_LOGIN_CUSTOM_CSS"></label>
		<div class="controls">
			<div class="custom-css-wrapper" data-bind="component: {
				name: 'TextArea',
				params: {
					value: loginCss,
					trigger: loginCss.trigger,
					width: 495,
					enable: capa,
					rows: 7
				}
			}"></div>
		</div>
	</div>
	<div class="control-group">
		<div class="controls">
			<div data-bind="component: {
				name: 'Checkbox',
				params: {
					enable: capa,
					label: 'TAB_BRANDING/LABEL_LOGIN_SHOW_POWERED_LINK',
					value: loginPowered
				}
			}"></div>
		</div>
	</div>
</div>

			</div>
			<div class="tab-pane" id="branding-user-section-id">

<div class="form-horizontal" data-bind="css: {'disabled-form': !capa()}">
	<div class="control-group">
		<label class="control-label" data-i18n="TAB_BRANDING/LABEL_USER_LOGO"></label>
		<div class="controls">
			<div data-bind="component: {
				name: 'Input',
				params: {
					value: userLogo,
					trigger: userLogo.trigger,
					placeholder: 'https://',
					size: 5,
					enable: capa
				}
			}"></div>
		</div>
	</div>
	<div class="control-group">
		<label class="control-label" data-i18n="TAB_BRANDING/LABEL_USER_LOGO_TITLE"></label>
		<div class="controls">
			<div data-bind="component: {
				name: 'Input',
				params: {
					value: userLogoTitle,
					trigger: userLogoTitle.trigger,
					size: 5,
					enable: capa
				}
			}"></div>
		</div>
	</div>
	<div class="control-group">
		<label class="control-label" data-i18n="TAB_BRANDING/LABEL_USER_LOGO_MESSAGE"></label>
		<div class="controls">
			<div data-bind="component: {
				name: 'Input',
				params: {
					value: userLogoMessage,
					trigger: userLogoMessage.trigger,
					placeholder: 'https://',
					size: 5,
					enable: capa
				}
			}"></div>
		</div>
	</div>
	<div class="control-group">
		<label class="control-label" data-i18n="TAB_BRANDING/LABEL_USER_CUSTOM_CSS"></label>
		<div class="controls">
			<div class="custom-css-wrapper" data-bind="component: {
				name: 'TextArea',
				params: {
					value: userCss,
					trigger: userCss.trigger,
					width: 495,
					enable: capa,
					rows: 7
				}
			}"></div>
		</div>
	</div>
</div>

			</div>
			<div class="tab-pane" id="branding-welcome-page-section-id">

<div class="form-horizontal" data-bind="css: {'disabled-form': !capa()}">
	<div class="control-group">
		<label class="control-label" data-i18n="TAB_BRANDING/LABEL_WELCOME_PAGE_URL"></label>
		<div class="controls">
			<div data-bind="component: {
				name: 'Input',
				params: {
					value: welcomePageUrl,
					trigger: welcomePageUrl.trigger,
					placeholder: 'https://',
					size: 5,
					enable: capa
				}
			}"></div>
		</div>
	</div>
	<div class="control-group">
	<label class="control-label" data-i18n="TAB_BRANDING/LABEL_WELCOME_PAGE_DISPLAY"></label>
		<div class="controls">
			<div data-bind="component: {
				name: 'Select',
				params: {
					options: welcomePageDisplay.options,
					value: welcomePageDisplay,
					trigger: welcomePageDisplay.trigger,
					optionsText: 'optText',
					optionsValue: 'optValue',
					size: 2,
					enable: capa
				}
			}"></div>
		</div>
	</div>
</div>

			</div>
		</div>
	</div>

	<div class="row" data-bind="visible: !capa() && !community">
		<div class="alert span8" style="margin-top: 10px;" data-i18n="[html]TAB_BRANDING/HTML_ALERT_PREMIUM"></div>
	</div>

</div>