<div class="e-component e-radio" data-bind="foreach: values">
	<label tabindex="0" data-bind="click: $parent.click, css: {'inline': $parent.inline}">
		<i role="radio" data-bind="css: $parent.value() === value ? 'icon-radio-checked' : 'icon-radio-unchecked'"></i>
		&nbsp;&nbsp;
		<span class="sub-label i18n" data-bind="attr: {'data-i18n': label}"></span>
	</label>
</div>