<?php

namespace SabreForRainLoop\CardDAV;

/**
 * IDirectory interface
 *
 * Implement this interface to have an addressbook marked as a 'directory'. A
 * directory is an (often) global addressbook.
 *
 * A full description can be found in the IETF draft:
 *   - draft-daboo-carddav-directory-gateway
 *
 * @copyright Copyright (C) 2007-2013 fruux GmbH (https://fruux.com/).
 * @author Evert Pot (http://evertpot.com/)
 * @license http://code.google.com/p/sabredav/wiki/License Modified BSD License
 */
interface IDirectory extends IAddressBook {


}
