<?php

namespace SabreForRainLoop\CalDAV\Schedule;

/**
 * Implement this interface to have a node be recognized as a CalDAV scheduling
 * outbox.
 *
 * @copyright Copyright (C) 2007-2013 fruux GmbH (https://fruux.com/).
 * @author Evert Pot (http://evertpot.com/)
 * @license http://code.google.com/p/sabredav/wiki/License Modified BSD License
 */
interface IOutbox extends \SabreForRainLoop\DAV\ICollection, \SabreForRainLoop\DAVACL\IACL {


}
