
.rl-strip-animation(@rgba, @stripOpacity) {
	background-image: linear-gradient(135deg, rgba(@rgba, @rgba, @rgba, @stripOpacity) 25%, transparent 25%,
		transparent 50%, rgba(@rgba, @rgba, @rgba, @stripOpacity) 50%, rgba(@rgba, @rgba, @rgba, @stripOpacity) 75%,
		transparent 75%, transparent);
}

@keyframes highlight-folder-row {
	0% { transform: scale(1); } 50% { transform: scale(1.1); } 100% { transform: scale(1); }
}

@keyframes bounce-me {
	0% { transform: scale(1); } 50% { transform: scale(1.2); } 100% { transform: scale(1); }
}

@keyframes textLoadingAnimationKeyFrame {
	0% { opacity: 1; } 33% { opacity: 0; } 100% { opacity: 1; }
}

@keyframes animate-stripes {
	0% {background-position: 0 0;} 100% {background-position: 60px 0;}
}

@keyframes login-form-shake  {
  0% {transform: translateX(0);}
  12.5% {transform: translateX(-6px) rotateY(-5deg)}
  37.5% {transform: translateX(5px) rotateY(4deg)}
  62.5% {transform: translateX(-3px) rotateY(-2deg)}
  87.5% {transform: translateX(2px) rotateY(1deg)}
  100% {transform: translateX(0)}
}

html.csstransitions.rl-started-trigger.no-mobile .b-login-content .loginFormWrapper {
	/*transform: scale(1.1);*/
	transform: translateY(-20px);
	opacity: 0.5;
}

/*html.rl-started-trigger.no-mobile #rl-content {
	opacity: 0.7;
}*/

#rl-loading {
	.transition(opacity 0.5s linear);
}

html.csstransitions.rl-started-delay {

	#rl-left {
		.transition(width 0.3s ease-out);
	}

	#rl-right {
		.transition(~'left 0.3s ease-out, right 0.3s ease-out');
	}

	#rl-sub-left, #rl-sub-left .messageList .inputSearch {
		.transition(width 0.3s ease-out);
	}

	#rl-sub-right {
		.transition(left 0.3s ease-out);
	}
}

html {
	&.rgba.cssanimations.backgroundsize .e-strip-animation {
		background-size: 60px 60px;
		.rl-strip-animation(0, 0.1);
		animation: animate-stripes 2s linear infinite;
	}
}

html.rl-anim {

	.rl-animated-inited {
		opacity: 1;
		.transition(opacity 0.5s linear);

		&.rl-animated-hidden {
			opacity: 0;
		}
	}

	&.cssanimations.csstransitions.no-mobile .b-login-content .errorAnimated {
		animation: login-form-shake 400ms ease-in-out;
	}

	&.no-mobile .b-login-content .errorAnimated .buttonLogin {
		color: #b94a48;
		font-weight: bold;
	}

	&.cssanimations.csstransitions.no-mobile .b-login-content .afterLoginHide {
		opacity: 0;
	}

	&.csstransitions.no-mobile .btn-group.dropdown.colored-toggle.open .animate-this-icon-on-open {
		animation: bounce-me .5s linear;
	}

	&.csstransitions.no-mobile .b-login-content .loginFormWrapper {
		.transition(all 0.3s ease-out);
	}

	&.csstransitions .button-delete-transitions {
		.transition(all 0.2s linear);
	}

	&.cssanimations .b-folders .e-item .anim-action-class {
		animation: highlight-folder-row 0.5s linear;
	}

	&.csstransitions .b-folders .btn.buttonContacts {
		.transition(margin 0.3s linear);
	}

	&.csstransitions .b-folders .b-content.opacity-on-panel-disabled {
		.transition(opacity 0.3s linear);
	}

	&.csstransitions .messageList {
		.messageListItem {
			.transition(max-height 400ms ease);
		}
		.listDragOver {
			.transition(all 400ms ease);
		}
	}

	&.csstransitions .b-list-content {
		.e-contact-item {
			.transition(max-height 400ms ease);
		}
	}

	&.csstransitions .modal.b-domain-content {
		.modal-body {
			.transition(left 500ms ease);
		}
	}

	&.csstransitions.rl-modal.rl-modal-animation .popups {
		overflow: hidden;
	}

	&.csstransitions .modal.fade {

		.transition(all 200ms ease-out);
		/*transform: scale(0.95);*/
		transform: translateY(-20px);

		&.in {
			transform: none;
		}
	}

	&.cssanimations .b-compose.loading .b-header-toolbar {
		background-size: 60px 60px;
		.rl-strip-animation(255, 0.2);
		animation: animate-stripes 2s linear infinite;
	}
}

.textLoadingAnimationD1, .textLoadingAnimationD2, .textLoadingAnimationD3 {
	animation: textLoadingAnimationKeyFrame 1s linear infinite 0s;
}

.textLoadingAnimationD2 {
	animation-delay: 0.3s;
}

.textLoadingAnimationD3 {
	animation-delay: 0.6s;
}
