<div class="popups">
	<div class="modal hide b-open-pgp-key-view-content" data-bind="modal: modalVisibility">
		<div>
			<div class="modal-header g-ui-user-select-none">
				<button type="button" class="close" data-bind="command: cancelCommand">&times;</button>
				<h3>
					<span class="i18n" data-i18n="POPUPS_VIEW_OPEN_PGP_KEY/TITLE_VIEW_OPEN_PGP_KEY"></span>
				</h3>
			</div>
			<div class="modal-body">
				<div class="form-horizontal">
					<div class="control-group">
						<pre class="key-viewer" data-bind="initDom: keyDom, text: key"></pre>
					</div>
				</div>
			</div>
			<div class="modal-footer">
				<a class="btn buttonClose" data-bind="command: cancelCommand">
					<i class="icon-remove"></i>
					&nbsp;&nbsp;
					<span class="i18n" data-i18n="POPUPS_VIEW_OPEN_PGP_KEY/BUTTON_CLOSE"></span>
				</a>
				<a class="btn buttonClose" data-bind="click: selectKey">
					<i class="icon-key"></i>
					&nbsp;&nbsp;
					<span class="i18n" data-i18n="POPUPS_VIEW_OPEN_PGP_KEY/BUTTON_SELECT"></span>
				</a>
			</div>
		</div>
	</div>
</div>
