<div class="b-settings-open-pgp g-ui-user-select-none">
	<div class="form-horizontal">
		<div class="legend">
			<span class="i18n" data-i18n="SETTINGS_OPEN_PGP/LEGEND_OPEN_PGP"></span>
		</div>
	</div>
	<button class="btn" data-bind="click: addOpenPgpKey">
		<i class="icon-list-add"></i>
		&nbsp;&nbsp;
		<span class="i18n" data-i18n="SETTINGS_OPEN_PGP/BUTTON_ADD_OPEN_PGP_KEY"></span>
	</button>
	&nbsp;&nbsp;
	<div style="display: inline-block" data-tooltip-join="left" data-bind="tooltip: isHttps ? '' : 'SETTINGS_OPEN_PGP/GENERATE_ONLY_HTTPS'">
		<button class="btn" data-bind="click: generateOpenPgpKey, disable: !isHttps">
			<i class="icon-key"></i>
			&nbsp;&nbsp;
			<span class="i18n" data-i18n="SETTINGS_OPEN_PGP/BUTTON_GENERATE_OPEN_PGP_KEYS"></span>
		</button>
	</div>
	<br />
	<br />
	<div class="control-group">
		<div class="controls">
			<div data-bind="component: {
				name: 'Checkbox',
				params: {
					label: 'SETTINGS_OPEN_PGP/LABEL_ALLOW_DRAFT_AUTOSAVE',
					value: allowDraftAutosave
				}
			}"></div>
		</div>
	</div>
	<br />
	<table class="table table-hover list-table" data-bind="i18nUpdate: openpgpkeys">
		<colgroup>
			<col />
			<col style="width: 1%" />
			<col style="width: 1%" />
		</colgroup>
		<tbody>
			<!-- ko foreach: openpgpkeysPrivate -->
			<tr class="open-pgp-key-item">
				<td>
					<span class="open-pgp-key-img i18n" data-i18n="[title]SETTINGS_OPEN_PGP/TITLE_PRIVATE">
						<i class="icon-lock"></i>
					</span>
					<span class="open-pgp-key-user">
					<!-- ko foreach: users -->
						<div class="open-pgp-key-user-address" data-bind="text: $data"></div>
					<!-- /ko -->
					</span>
					<a class="btn btn-small btn-small-small btn-danger pull-right button-delete button-delete-transitions" data-bind="css: {'delete-access': deleteAccess()}, click: function(openPgpKey) { $root.deleteOpenPgpKey(openPgpKey); }">
						<span class="i18n" data-i18n="SETTINGS_OPEN_PGP/DELETING_ASK"></span>
					</a>
				</td>
				<td>
					<span class="delete-open-pgp-key" data-bind="visible: !deleteAccess(), click: function (openPgpKey) { $root.openPgpKeyForDeletion(openPgpKey); }">
						<i class="icon-trash"></i>
					</span>
				</td>
				<td>
					<span class="view-open-pgp-key" data-bind="click: function (openPgpKey) { $root.viewOpenPgpKey(openPgpKey); }">
						<i class="icon-eye"></i>
					</span>
				</td>
			</tr>
			<!-- /ko -->
			<!-- ko foreach: openpgpkeysPublic -->
			<tr class="open-pgp-key-item">
				<td>
					<span class="open-pgp-key-img i18n" data-i18n="[title]SETTINGS_OPEN_PGP/TITLE_PUBLIC">
						<i class="icon-key"></i>
					</span>
					<span class="open-pgp-key-user">
					<!-- ko foreach: users -->
						<div class="open-pgp-key-user-address">
							<span data-bind="text: $data"></span> (<span class="open-pgp-key-id" data-bind="text: $parent.id"></span>)
						</div>
					<!-- /ko -->
					</span>
					<a class="btn btn-small btn-small-small btn-danger pull-right button-delete button-delete-transitions" data-bind="css: {'delete-access': deleteAccess()}, click: function(oOpenPGP) { $root.deleteOpenPgpKey(oOpenPGP); }">
						<span class="i18n" data-i18n="SETTINGS_OPEN_PGP/DELETING_ASK"></span>
					</a>
				</td>
				<td>
					<span class="delete-open-pgp-key" data-bind="visible: !deleteAccess(), click: function (openPgpKey) { $root.openPgpKeyForDeletion(openPgpKey); }">
						<i class="icon-trash"></i>
					</span>
				</td>
				<td>
					<span class="view-open-pgp-key" data-bind="click: function (openPgpKey) { $root.viewOpenPgpKey(openPgpKey); }">
						<i class="icon-eye"></i>
					</span>
				</td>
			</tr>
			<!-- /ko -->
		</tbody>
	</table>
</div>
