<div class="b-settings-general g-ui-user-select-none">
	<div class="form-horizontal">
		<div class="legend">
			<span class="i18n i18n-animation" data-i18n="SETTINGS_GENERAL/LEGEND_GENERAL"></span>
		</div>
		<div class="control-group">
			<label class="control-label">
				<span class="i18n" data-i18n="SETTINGS_GENERAL/LABEL_IDENTITY"></span>
			</label>
			<div class="controls">
				<div style="display: inline-block; margin-top: 5px">
					<b data-bind="click: editMainIdentity, text: identityMainDesc"
						style="border-bottom: 1px dashed #555; cursor: pointer; padding: 2px 0px;"></b>
				</div>
			</div>
		</div>
		<div class="control-group" data-bind="visible: allowLanguagesOnSettings">
			<label class="control-label">
				<span class="i18n" data-i18n="SETTINGS_GENERAL/LABEL_LANGUAGE"></span>
			</label>
			<div class="controls">
				<div class="flag-selector">
					<span class="flag-wrapper">
						<span data-bind="css: 'flag flag-' + language().toLowerCase()" style=""></span>
					</span>
					<span class="flag-name" tabindex="0" data-bind="text: languageFullName, click: selectLanguage, onSpace: selectLanguage, onEnter: selectLanguage"></span>
					&nbsp;&nbsp;
					<div data-bind="component: {
						name: 'SaveTrigger',
						params: { value: languageTrigger }
					}"></div>
				</div>
			</div>
		</div>
		<div class="control-group hide-on-mobile">
			<label class="control-label">
			   <span class="i18n" data-i18n="SETTINGS_GENERAL/LABEL_LAYOUT"></span>
			</label>
			<div class="controls">
				<div data-bind="component: {
					name: 'Select',
					params: {
						options: layoutTypes,
						value: layout,
						trigger: layoutTrigger,
						optionsText: 'name',
						optionsValue: 'id'
					}
				}"></div>
			</div>
		</div>
		<div class="control-group">
			<label class="control-label">
			   <span class="i18n" data-i18n="SETTINGS_GENERAL/LABEL_EDITOR"></span>
			</label>
			<div class="controls">
				<div data-bind="component: {
					name: 'Select',
					params: {
						options: editorDefaultTypes,
						value: editorDefaultType,
						trigger: editorDefaultTypeTrigger,
						optionsText: 'name',
						optionsValue: 'id'
					}
				}"></div>
			</div>
		</div>
		<div class="control-group">
			<label class="control-label">
			   <span class="i18n" data-i18n="SETTINGS_GENERAL/LABEL_MESSAGE_PER_PAGE"></span>
			</label>
			<div class="controls">
				<div data-bind="component: {
					name: 'Select',
					params: {
						options: messagesPerPageArray,
						value: messagesPerPage,
						trigger: mppTrigger,
						size: 2,
						width: 80
					}
				}"></div>
			</div>
		</div>
		<div class="control-group">
			<div class="controls">
				<div data-bind="component: {
					name: 'Checkbox',
					params: {
						label: 'SETTINGS_GENERAL/LABEL_SHOW_IMAGES',
						value: showImages
					}
				}"></div>
				<div class="hide-on-mobile" data-bind="component: {
					name: 'Checkbox',
					params: {
						label: 'SETTINGS_GENERAL/LABEL_USE_CHECKBOXES_IN_LIST',
						value: useCheckboxesInList
					}
				}"></div>
				<br />
				<div data-bind="visible: threadsAllowed, component: {
					name: 'Checkbox',
					params: {
						label: 'SETTINGS_GENERAL/LABEL_USE_THREADS',
						value: useThreads
					}
				}"></div>
				<div data-bind="component: {
					name: 'Checkbox',
					params: {
						label: 'SETTINGS_GENERAL/LABEL_REPLY_SAME_FOLDER',
						value: replySameFolder
					}
				}"></div>
			</div>
		</div>
	</div>
	<div class="form-horizontal hide-on-mobile" data-bind="visible: isDesktopNotificationSupported() || soundNotificationIsSupported()">
		<div class="legend">
			<span class="i18n i18n-animation" data-i18n="SETTINGS_GENERAL/LABEL_NOTIFICATIONS"></span>
		</div>
		<div class="control-group">
			<div class="controls">
				<div data-bind="visible: isDesktopNotificationSupported">
					<div data-bind="component: {
						name: 'Checkbox',
						params: {
							label: 'SETTINGS_GENERAL/LABEL_CHROME_NOTIFICATION_DESC',
							value: enableDesktopNotification,
							disable: isDesktopNotificationDenied,
							inline: true
						}
					}"></div>
					&nbsp;
					<span data-bind="visible: isDesktopNotificationDenied">
						<span class="i18n" style="color: #999" data-i18n="SETTINGS_GENERAL/LABEL_CHROME_NOTIFICATION_DESC_DENIED"></span>
					</span>
				</div>
				<div data-bind="visible: soundNotificationIsSupported">
					<div data-bind="component: {
						name: 'Checkbox',
						params: {
							label: 'SETTINGS_GENERAL/LABEL_SOUND_NOTIFICATION',
							value: enableSoundNotification,
							inline: true
						}
					}"></div>
					&nbsp;&nbsp;
					<span data-bind="click: testSoundNotification" style="color:green;cursor:pointer">
						<i class="icon-right-dir iconsize20"></i>
					</span>
				</div>
			</div>
		</div>
	</div>
</div>