
.b-settings-general {

	html.rl-mobile &{
		margin-right: 15px;
	}

	.notification-desc-denied {
		color: #999;
		display: none;
	}

	.denied-by-browser {
		.notification-desc-denied {
			display: inline;
		}
		.notification-desc {
			color: #999;
		}
	}

	.flag-selector {
		padding-top: 5px;
	}

	.flag-name {

		border-bottom: 1px dashed #555;
		cursor: pointer;
		padding: 2px 0px;

		&:focus {
			outline: 1px;
			outline-style: dotted;
		}
	}
}
