<div class="popups">
	<div class="modal b-domain-alias-content" data-bind="modal: modalVisibility">
		<div class="modal-header">
			<button type="button" class="close" data-bind="command: cancelCommand">&times;</button>
			<h3 data-i18n="POPUPS_DOMAIN_ALIAS/TITLE_ADD_DOMAIN_ALIAS"></h3>
		</div>
		<div class="modal-body">
			<div class="control-group" data-bind="css: {error: '' !== savingError()}">
				<label class="control-label" data-i18n="POPUPS_DOMAIN_ALIAS/LABEL_ALIAS"></label>
				<div class="controls">
					<div data-bind="component: {
						name: 'Input',
						params: {
							value: name,
							size: 4
						}
					}"></div>
					<div data-bind="visible: '' !== savingError()">
						<span class="error-desc" data-bind="text: savingError"></span>
					</div>
				</div>
			</div>
			<div class="control-group">
				<label class="control-label" data-i18n="POPUPS_DOMAIN_ALIAS/LABEL_DOMAIN"></label>
				<div class="controls">
					<div data-bind="component: {
						name: 'Select',
						params: {
							options: domainsOptions,
							value: alias,
							optionsText: 'optText',
							optionsValue: 'optValue',
							size: 4
						}
					}"></div>
				</div>
			</div>
		</div>
		<div class="modal-footer">
			<a class="btn buttonClose" data-bind="command: cancelCommand">
				<i class="icon-remove"></i>
				&nbsp;&nbsp;
				<span data-i18n="POPUPS_DOMAIN_ALIAS/BUTTON_CLOSE"></span>
			</a>
			<a class="btn buttonClear" data-bind="command: createCommand">
				<i class="icon-plus" data-bind="visible: !saving()"></i>
				<i class="icon-spinner animated" data-bind="visible: saving"></i>
				&nbsp;&nbsp;
				<span data-i18n="POPUPS_DOMAIN_ALIAS/BUTTON_ADD"></span>
			</a>
		</div>
	</div>
</div>