<div class="popups">
	<div class="modal hide b-plugin-content" data-bind="modal: modalVisibility">
		<div class="modal-header">
			<button type="button" class="close" data-bind="command: cancelCommand">&times;</button>
			<h4>
				<span data-i18n="POPUPS_PLUGIN/TITLE_PLUGIN"></span>
				&nbsp;
				"<span data-bind="text: name"></span>"
			</h4>
		</div>
		<div class="modal-body">
			<form class="form-horizontal plugin-form" action="#/" onsubmit="return false;">
				<div class="well" data-bind="visible: !hasConfiguration()">
					<span data-i18n="POPUPS_PLUGIN/DESC_NOTHING_TO_CONFIGURE"></span>
				</div>
				<div class="alert" data-bind="visible: '' !== saveError()">
					<button type="button" class="close" data-bind="click: function () { saveError('') }">&times;</button>
					<span data-bind="text: saveError"></span>
				</div>
				<div data-bind="foreach: configures, visible: hasConfiguration">
					<div data-bind="template: { name: 'AdminSettingsPluginProperty' }"></div>
				</div>
			</form>
		</div>
		<div class="modal-footer">
			<span class="information pull-left" data-bind="visible: hasReadme, popover: readmePopoverConf">
				<i class="icon-info"></i>
			</span>
			<a class="btn buttonClose" data-bind="command: cancelCommand">
				<i class="icon-remove"></i>
				&nbsp;&nbsp;
				<span data-i18n="POPUPS_PLUGIN/BUTTON_CLOSE"></span>
			</a>
			<a class="btn buttonClear" data-bind="command: saveCommand, visible: hasConfiguration">
				<i class="icon-ok"></i>
				&nbsp;&nbsp;
				<span data-i18n="POPUPS_PLUGIN/BUTTON_SAVE"></span>
			</a>
		</div>
	</div>
</div>