<div class="b-settings-security">
	<div class="form-horizontal" data-bind="visible: capaAutoLogout">
		<div class="legend">
			<span class="i18n" data-i18n="SETTINGS_SECURITY/LEGEND_SECURITY"></span>
		</div>
		<div class="control-group">
			<label class="control-label">
			   <span class="i18n" data-i18n="SETTINGS_SECURITY/LABEL_AUTOLOGOUT"></span>
			</label>
			<div class="controls">
				<div data-bind="component: {
					name: 'Select',
					params: {
						inline: true,
						options: autoLogoutOptions,
						value: autoLogout,
						trigger: autoLogout.trigger,
						optionsText: 'name',
						optionsValue: 'id'
					}
				}"></div>
			</div>
		</div>
		<br />
		<div class="control-group" data-bind="visible: capaTwoFactor">
			<label class="control-label"></label>
			<div class="controls">
				<i class="icon-lock" />
				&nbsp;
				<span class="i18n g-ui-link" tabindex="0" data-i18n="SETTINGS_SECURITY/LABEL_CONFIGURE_TWO_FACTOR" data-bind="click: configureTwoFactor, onSpace: configureTwoFactor, onEnter: configureTwoFactor"></span>
			</div>
		</div>
	</div>
</div>