<tr class="e-item" data-bind="css: {'disabled': disabled }">
	<td class="configure-plugin-action e-action">
		<i class="plugin-img icon-bolt"></i>
		<span class="plugin-name" data-bind="text: name"></span>
	</td>
	<td>
		<span class="disabled-plugin">
			<i data-bind="css: {'e-action icon-checkbox-checked': !disabled(), 'e-action icon-checkbox-unchecked': disabled}"></i>
		</span>
	</td>
</tr>