<div class="e-item" data-bind="visible: visible, css: { 'i-am-inbox-wrapper': isInbox }">
	<a class="e-link" data-bind="droppable: function (oEvent, oUi) { $root.messagesDrop($data, oUi); },
	   droppableOver: function (oEvent, oUi) { $root.messagesDropOver($data, oUi); },
	   droppableOut: function (oEvent, oUi) { $root.messagesDropOut($data, oUi); },
	   css: { 'i-am-inbox': isInbox, 'selected': selected() && !isSystemFolder(), 'selectable': selectableForFolderList, 'hidden' : hidden, 'print-count': hasUnreadMessages, 'unread-sub': hasSubScribedUnreadMessagesSubfolders, 'system': isSystemFolder, 'anim-action-class': actionBlink }">
		<span class="badge pull-right count" data-bind="text: printableUnreadCount"></span>
		<i data-bind="css: collapsedCss()"></i>
		<span class="focused-poiner"></span>
		<span class="name" data-bind="text: name"></span>
	</a>
	<div class="b-sub-folders" data-bind="template: { name: 'MailFolderListItem', foreach: subFolders }, css: { 'collapsed': collapsed() }"></div>
</div>