<div class="b-login-content">
	<div class="loginFormWrapper" data-bind="css: {'afterLoginHide': formHidden}">
		<center>
			<div class="alert alertError" data-bind="visible: '' !== submitError()">
				<button type="button" class="close" data-bind="click: function () { submitError('') }">&times;</button>
				<span data-bind="text: submitError"></span>
			</div>
			<div class="wrapper-parent">
				<form class="wrapper submitting-pane loginForm thm-login thm-login-text" action="#/" data-bind="submit: submitForm, css: {'errorAnimated': formError, 'submitting': submitRequest()}">
					<div class="controls" data-bind="css: {'error': loginError, 'animated': loginErrorAnimation}">
						<div class="input-append">
							<input type="text" class="input-block-level inputLogin checkAutocomplete"
								name="RainLoopAdminLogin" id="RainLoopAdminLogin"
								style="padding-right: 35px;"
								autocorrect="off" autocapitalize="off" spellcheck="false" data-i18n="[placeholder]LOGIN/LABEL_LOGIN"
								data-bind="textInput: login, hasFocus: loginFocus, disable: submitRequest" />
							<span class="add-on">
								<i class="icon-user"></i>
							</span>
						</div>
					</div>
					<div class="controls" data-bind="css: {'error': passwordError, 'animated': passwordErrorAnimation}">
						<div class="input-append">
							<input type="password" class="input-block-level inputPassword checkAutocomplete"
								placeholder="Password" name="RainLoopAdminPassword" id="RainLoopAdminPassword"
								style="padding-right: 35px;"
								autocorrect="off" autocapitalize="off" spellcheck="false" data-i18n="[placeholder]LOGIN/LABEL_PASSWORD"
								data-bind="textInput: password, disable: submitRequest" />
							<span class="add-on" tabindex="-1" data-tooltip-join="left" data-bind="command: submitCommand, tooltip: 'LOGIN/BUTTON_LOGIN'">
								<i class="icon-key" data-bind="visible: '' === password()"></i>
								<button type="submit" class="btn-submit-icon-wrp" data-bind="visible: '' !== password()">
									<i class="icon-right-middle login-submit-icon"></i>
								</button>
							</span>
						</div>
					</div>
					<div class="controls controls-submit-wrp" data-bind="css: {'display-none': hideSubmitButton}">
						<button type="submit" class="btn btn-large btn-block buttonLogin" data-bind="command: submitCommand">
							<span data-i18n="LOGIN/BUTTON_LOGIN"></span>
						</button>
					</div>
				</form>
			</div>
			<div class="e-powered thm-powered" data-bind="visible: loginPowered">
				Powered by <a href="http://www.rainloop.net" target="_blank">RainLoop</a>
			</div>
		</center>
	</div>
	<div class="loginAfter"></div>
<div>