<div class="control-group" style="margin-bottom: 0">
	<div class="controls">
		<div data-bind="component: {
			name: 'Select',
			params: {
				options: [1, 2, 3, 7, 30],
				label: 'POPUPS_FILTER/REPLY_INTERVAL_LABEL',
				value: actionValueThird,
				size: 1
			}
		}"></div>
	</div>
</div>
<div class="control-group" data-bind="css: {'error': actionValueFourth.error}" style="margin-bottom: 0">
	<div class="controls">
		<input type="text" class="span5 i18n" data-bind="value: actionValueFourth"
			data-i18n="[placeholder]POPUPS_FILTER/VACATION_RECIPIENTS_LABEL" />
		<button class="btn setRecipientsBtn" data-bind="click: setRecipients">
			<i class="icon-plus"></i>
		</button>
	</div>
</div>
<div class="control-group" style="margin-bottom: 0">
	<div class="controls">
		<input type="text" class="span5 i18n" data-bind="value: actionValueSecond"
			data-i18n="[placeholder]POPUPS_FILTER/VACATION_SUBJECT_LABEL" />
	</div>
</div>
<div class="control-group" data-bind="css: {'error': actionValue.error}" style="margin-bottom: 0">
	<div class="controls">
		<textarea class="span5 i18n" data-bind="value: actionValue" style="height: 100px;"
			data-i18n="[placeholder]POPUPS_FILTER/VACATION_MESSAGE_LABEL"></textarea>
	</div>
</div>
<div class="control-group">
	<div class="controls">
		<div data-bind="component: {
			name: 'Checkbox',
			params: {
				label: 'POPUPS_FILTER/STOP_LABEL',
				value: actionNoStop
			}
		}"></div>
	</div>
</div>
