<tr class="e-item" data-bind="css: {'disabled': disabled }">
	<td class="e-action">
		<span class="domain-name" data-bind="text: name"></span>
		<span class="domain-alias" data-bind="if: alias">(alias)</span>
	</td>
	<td>
		<a class="btn btn-small btn-small-small btn-danger pull-right button-delete button-delete-transitions" data-bind="css: {'delete-access': deleteAccess()}, click: function(oDomain) { $root.deleteDomain(oDomain); }">
			<span data-i18n="TAB_DOMAINS/DELETE_ARE_YOU_SURE"></span>
		</a>
	</td>
	<td>
		<span class="delete-domain" data-bind="visible: !deleteAccess(), click: function (oDomain) { $root.domainForDeletion(oDomain); }">
			<i class="icon-trash"></i>
		</span>
	</td>
	<td>
		<span class="disabled-domain" data-bind="click: function (oDomain) { $root.disableDomain(oDomain); }">
			<i data-bind="css: {'icon-checkbox-checked': !disabled(), 'icon-checkbox-unchecked': disabled}"></i>
		</span>
	</td>
</tr>