<div class="popups">
	<div class="modal hide b-compose" data-backdrop="static" data-bind="modal: modalVisibility, css: {'loading': saving() || sending()}">
		<div class="modal-header b-header-toolbar g-ui-user-select-none">
			<a class="btn btn-large button-send" data-bind="command: sendCommand, tooltipErrorTip: sendErrorDesc, css: {'btn-success': sendButtonSuccess, 'btn-danger': sendError, 'btn-warning': sendSuccessButSaveError }">
				<i class="icon-white" data-bind="css: {'icon-paper-plane': !sending(), 'icon-spinner animated big': sending()}"></i>
				<span class="i18n btn-text hide-on-mobile" data-i18n="COMPOSE/BUTTON_SEND"></span>
			</a>
			<a class="btn button-save" data-bind="visible: allowFolders, command: saveCommand, tooltipErrorTip: savedErrorDesc, css: {'btn-danger': savedError }">
				<i data-bind="css: {'icon-floppy': !saving(), 'icon-spinner animated': saving(), 'icon-white': savedError()}"></i>
				<span class="i18n btn-text hide-on-mobile" data-i18n="COMPOSE/BUTTON_SAVE"></span>
			</a>

			<a class="close-custom" data-tooltip-join="top" data-bind="click: tryToClosePopup, tooltip: 'COMPOSE/BUTTON_CANCEL'">&times;</a>
			<a class="minimize-custom" data-tooltip-join="top" data-bind="click: skipCommand, tooltip: 'COMPOSE/BUTTON_MINIMIZE'"></a>

			<a class="btn btn-danger button-delete button-delete-transitions" data-bind="command: deleteCommand">
				<i class="icon-trash icon-white"></i>
			</a>
			<span class="saved-text hide-on-mobile" data-bind="text: savedTimeText"></span>
		</div>
		<div class="modal-body">
			<div>
				<div class="b-header g-ui-user-select-none">
					<div class="g-ui-table">
						<div class="e-row" style="height: 40px;">
							<div class="e-cell e-label">
								<label class="control-label">
									<span class="i18n" data-i18n="COMPOSE/TITLE_FROM"></span>
								</label>
							</div>
							<div class="e-cell e-value">
								<div class="dropdown pull-left" data-bind="registrateBootstrapDropdown: true, openDropdownTrigger: identitiesDropdownTrigger">
									<a class="dropdown-toggle g-ui-dropdown e-identity" href="#" tabindex="-1"
										id="identity-label-id" role="button" data-toggle="dropdown"
										data-bind="text: currentIdentityView, dropdownCloser: true, css: {'multiply': 1 < identitiesOptions().length }">
									</a>
									<!-- ko if: 1 < identitiesOptions().length -->
										<ul class="dropdown-menu g-ui-menu" role="menu" aria-labelledby="identity-label-id">
											<!-- ko foreach: identitiesOptions -->
												<li class="e-item" role="presentation">
													<a class="e-link menuitem" tabindex="-1" href="javascript:void(0);" data-bind="click: function (oIdentity) { $root.selectIdentity(oIdentity); return true; }">
														<span data-bind="text: optText"></span>
													</a>
												</li>
											<!-- /ko -->
										</ul>
									<!-- /ko -->
								</div>
								<div class="pull-right">
									<div class="btn-group dropdown colored-toggle pull-right" style="margin-right: 4px;">
										<a class="btn single dropdown-toggle buttonMore" data-toggle="dropdown">
											<i class="icon-list"></i>
										</a>
										<ul class="dropdown-menu g-ui-menu" role="menu">
											<li class="e-item" data-bind="click: function () { requestReadReceipt(!requestReadReceipt()); }">
												<a class="e-link">
													<i class="icon-checkbox-unchecked" data-bind="css: {'icon-checkbox-checked': requestReadReceipt(), 'icon-checkbox-unchecked': !requestReadReceipt() }"></i>
													&nbsp;&nbsp;
													<span class="i18n" data-i18n="COMPOSE/BUTTON_REQUEST_READ_RECEIPT"></span>
												</a>
											</li>
											<li class="e-item" data-bind="click: function () { requestDsn(!requestDsn()); }">
												<a class="e-link">
													<i class="icon-checkbox-unchecked" data-bind="css: {'icon-checkbox-checked': requestDsn(), 'icon-checkbox-unchecked': !requestDsn() }"></i>
													&nbsp;&nbsp;
													<span class="i18n" data-i18n="COMPOSE/BUTTON_REQUEST_DSN"></span>
												</a>
											</li>
											<li class="e-item" data-bind="click: function () { markAsImportant(!markAsImportant()); }">
												<a class="e-link">
													<i class="icon-checkbox-unchecked" data-bind="css: {'icon-checkbox-checked': markAsImportant(), 'icon-checkbox-unchecked': !markAsImportant() }"></i>
													&nbsp;&nbsp;
													<span class="i18n" data-i18n="COMPOSE/BUTTON_MARK_AS_IMPORTANT"></span>
												</a>
											</li>
											<li class="divider" data-bind="visible: capaOpenPGP"></li>
											<li class="e-item" data-bind="visible: capaOpenPGP, click: openOpenPgpPopup, css: {'disabled': isHtml()}">
												<a class="e-link">
													<i class="icon-key"></i>
													&nbsp;&nbsp;
													<span class="i18n" data-i18n="COMPOSE/BUTTON_OPEN_PGP"></span>
												</a>
											</li>
										</ul>
									</div>
									<div class="btn-group pull-right">&nbsp;</div>
									<div class="btn-group pull-right">
										<a class="btn single" data-tooltip-join="top" data-bind="visible: allowContacts, command: contactsCommand, tooltip: 'FOLDER_LIST/BUTTON_CONTACTS'">
											<i class="icon-address-book"></i>
										</a>
									</div>
									<div class="btn-group pull-right">&nbsp;</div>
								</div>
								<span class="pull-right">
									<span class="i18n g-ui-link" data-i18n="COMPOSE/TITLE_CC"
										data-bind="visible: !showCc(), click: function () { showCc(true); }"></span>
									&nbsp;&nbsp;
									<span data-bind="visible: !showBcc()">
										<span class="i18n g-ui-link" data-i18n="COMPOSE/TITLE_BCC"
											data-bind="click: function () { showBcc(true); }"></span>
										&nbsp;&nbsp;
									</span>
									<span class="i18n g-ui-link" data-i18n="COMPOSE/TITLE_REPLY_TO"
										data-bind="visible: !showReplyTo(), click: function () { showReplyTo(true); }"></span>
									&nbsp;
								</span>
							</div>
						</div>
						<div class="e-row">
							<div class="e-cell e-label">
								<label class="control-label" data-bind="css: {'error-to': emptyToError}">
									<span class="i18n" data-i18n="COMPOSE/TITLE_TO" data-tooltip-join="top"
										data-bind="tooltipErrorTip: emptyToErrorTooltip"></span>
								</label>
							</div>
							<div class="e-cell e-value">
								<input type="text" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false" data-bind="emailsTags: to, emailsTagsFocus: to.focused, autoCompleteSource: emailsSource" />
							</div>
						</div>
						<div class="e-row cc-row" data-bind="visible: showCc">
							<div class="e-cell e-label">
								<span class="i18n" data-i18n="COMPOSE/TITLE_CC"></span>
							</div>
							<div class="e-cell e-value">
								<input type="text" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false" data-bind="emailsTags: cc, autoCompleteSource: emailsSource" />
							</div>
						</div>
						<div class="e-row bcc-row" data-bind="visible: showBcc">
							<div class="e-cell e-label">
								<span class="i18n" data-i18n="COMPOSE/TITLE_BCC"></span>
							</div>
							<div class="e-cell e-value">
								<input type="text" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false" data-bind="emailsTags: bcc, autoCompleteSource: emailsSource" />
							</div>
						</div>
						<div class="e-row reply-to-row" data-bind="visible: showReplyTo">
							<div class="e-cell e-label">
								<span class="i18n" data-i18n="COMPOSE/TITLE_REPLY_TO"></span>
							</div>
							<div class="e-cell e-value">
								<input type="text" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false" data-bind="emailsTags: replyTo, autoCompleteSource: emailsSource" />
							</div>
						</div>
						<div class="e-row">
							<div class="e-cell e-label">
								<span class="i18n" data-i18n="COMPOSE/TITLE_SUBJECT"></span>
							</div>
							<div class="e-cell e-value">
								<input type="text" size="70" autocomplete="off" data-bind="textInput: subject, hasFocus: subject.focused" />
							</div>
						</div>
						<div class="e-row">
							<div class="e-cell e-label"></div>
							<div class="e-cell e-value">
								<div>
									<div class="pull-left">
										<div class="btn-group" data-toggle="buttons-radio">
											<button type="button" class="btn first" data-bind="click: function () { attachmentsPlace(false); },
												css: { 'active': !attachmentsPlace() }">
												<i class="icon-file-text"></i>
											</button>
											<button type="button" class="btn last" data-tooltip-join="left" data-bind="click: function () { attachmentsPlace(true); },
												css: { 'btn-danger': 0 < attachmentsInErrorCount(), 'active': attachmentsPlace() },
												tooltipErrorTip: attachmentsErrorTooltip">
												<span data-bind="visible: 0 < attachmentsCount()">
													<b data-bind="text: attachmentsCount"></b>
													&nbsp;&nbsp;
												</span>
												<i data-bind="css: { 'icon-attachment': 0 === attachmentsInProcessCount(), 'icon-spinner animated': 0 < attachmentsInProcessCount(), 'icon-white': 0 < attachmentsInErrorCount() }"></i>
											</button>
										</div>
									</div>
									<div class="pull-right" style="margin-right: 4px;">
										<div class="btn-group pull-right">
											<a class="btn first" data-tooltip-join="bottom"
												style="padding-left: 10px; padding-right: 10px;"
												data-bind="visible: addAttachmentEnabled(), initDom: composeUploaderButton, tooltip: 'COMPOSE/ATTACH_FILES',
													css: {'last': !(driveEnabled() && driveVisible()) && !(dropboxEnabled())}">
												<sup style="font-weight: bold; font-size: 100%; top: -0.3em;">+</sup><i class="icon-attachment"></i>
											</a>
											<a class="btn" data-tooltip-join="bottom"
												data-bind="visible: dropboxEnabled, command: dropboxCommand, tooltip: 'COMPOSE/DROPBOX',
													css: {'first': !addAttachmentEnabled(), 'last': !(driveEnabled() && driveVisible())}">
												<i class="icon-dropbox"></i>
											</a>
											<a class="btn last" data-tooltip-join="bottom"
												data-bind="visible: driveEnabled() && driveVisible(), command: driveCommand, tooltip: 'COMPOSE/GOOGLE_DRIVE',
													css: {'first': !addAttachmentEnabled() && !(driveEnabled() && driveVisible())}">
												<i class="icon-google-drive"></i>
											</a>
										</div>
									</div>
								</div>
							</div>
						</div>
					</div>
				</div>

				<div class="attachmentAreaParent b-content" style="height: 200px; min-height: 200px" data-bind="nano: true, scrollerShadows: true, visible: attachmentsPlace, initResizeTrigger: [resizer(), 200, 57]">
					<div class="content g-scrollbox" data-scroller-shadows-content>
						<div class="content-wrapper">
							<div class="b-attachment-place" data-bind="visible: addAttachmentEnabled() && dragAndDropEnabled() && dragAndDropVisible(), initDom: composeUploaderDropPlace, css: {'dragAndDropOver': dragAndDropOver}">
								<span class="i18n" data-i18n="COMPOSE/ATTACH_DROP_FILES_DESC"></span>
							</div>
							<ul class="attachmentList" data-bind="template: { name: 'ComposeAttachment', foreach: attachments }"></ul>
							<div class="no-attachments-desc" data-bind="visible: 0 === attachments().length">
								<span class="i18n" data-i18n="COMPOSE/NO_ATTACHMENTS_HERE_DESC"></span>
							</div>
						</div>
					</div>
				</div>

				<div class="textAreaParent" style="height: 200px; min-height: 200px"
					data-bind="visible: !attachmentsPlace(), initDom: composeEditorArea, initResizeTrigger: [resizer(), 200, 40]"></div>
			</div>
		</div>
	</div>
</div>
