
.b-settings-content {
	.b-settings-themes {
		.legend {
			width: 100%;
		}
	}
}

html.rl-mobile .b-settings-content .b-settings-theme .legend {
	width: auto;
}

.b-themes-list {

	html.rl-mobile &{
		margin-right: 15px;
	}

	.e-item {

		display: inline-block;
		border: 2px solid transparent;
		cursor: pointer;

		color: #000;
		background-color: #fff;
		padding: 16px;
		margin: 5px;

		&:hover {
			border: 2px solid grey;
		}

		&.selected {
			background-color: #eee;
			border: 2px solid #000;
		}

		.e-image {
			width: 100px;
			height: 100px;
			border: 1px solid #ddd;
		}
	}
}
