.popups {
	.b-filter-content {

		width: 750px;

		.modal-header {
			background-color: #fff;
		}

		.button-delete {
			cursor: pointer;
		}

		.setRecipientsBtn {
			margin-top: -2px;
			margin-left: 5px;
		}
	}
}
