<?php

namespace SabreForRainLoop\VObject\Property\ICalendar;

use
    SabreForRainLoop\VObject\Property,
    SabreForRainLoop\VObject\Parser\MimeDir,
    SabreForRainLoop\VObject\DateTimeParser,
    SabreForRainLoop\VObject\TimeZoneUtil;

/**
 * DateTime property
 *
 * This object represents DATE values, as defined here:
 *
 * http://tools.ietf.org/html/rfc5545#section-3.3.5
 *
 * @copyright Copyright (C) 2007-2013 fruux GmbH. All rights reserved.
 * @author Evert Pot (http://evertpot.com/)
 * @license http://code.google.com/p/sabredav/wiki/License Modified BSD License
 */
class Date extends DateTime {

}
