<div class="b-admin-domains g-ui-user-select-none">
	<div class="legend" data-i18n="TAB_DOMAINS/LEGEND_DOMAINS"></div>
	<div>
		<a class="btn" data-bind="click: createDomain">
			<i class="icon-plus"></i>
			&nbsp;&nbsp;
			<span data-i18n="TAB_DOMAINS/BUTTON_ADD_DOMAIN"></span>
		</a>
		&nbsp;&nbsp;
		<a class="btn" data-bind="click: createDomainAlias">
			<i class="icon-plus"></i>
			&nbsp;&nbsp;
			<span data-i18n="TAB_DOMAINS/BUTTON_ADD_ALIAS"></span>
		</a>
		<br />
		<br />
		<blockquote>
			<p class="muted" data-i18n="[html]TAB_DOMAINS/HTML_DOMAINS_HELPER"></p>
		</blockquote>
		<table class="table table-hover b-admin-domains-list-table" data-bind="i18nUpdate: domains">
			<colgroup>
				<col />
				<col style="width: 140px" />
				<col style="width: 1%" />
				<col style="width: 1%" />
			</colgroup>
			<tbody data-bind="template: { name: 'AdminSettingsDomainListItem', foreach: domains }"></tbody>
		</table>
	</div>
</div>