
.b-settings-folders {

	html.rl-mobile &{
		margin-right: 35px;
	}

	&.ignore-folder-subscribe {
		.subscribe-folder, .unsubscribe-folder {
			display: none;
		}
	}

	.process-place {
		text-align: center;
		width: 600px;
		padding: 14px 0;
	}

	.folders-list-error {
		width: 550px;
		margin: 10px 0px;
	}

	.list-table {

		width: 600px;

		.e-action {
			cursor: pointer;
		}

		td {
			padding: 4px 8px;
			line-height: 30px;
		}

		.folder-padding {
			display: inline-block;
			width: 0;
		}

		.folder-name {
			display: inline-block;
			word-break: break-all;
			white-space: pre-wrap;
			box-sizing: border-box;
			margin-left: 7px;
			line-height: 22px;
			cursor: default;
		}

		.folder-system-name {
			display: inline-block;
			line-height: 22px;
			color: #999;
			cursor: default;
		}

		.folder-name.can-be-edited:hover {
			border-bottom: 1px dashed #333;
			cursor: pointer;
		}

		.folder-name-input {
			border-width: 1px;
			margin-bottom: 0;
			margin-left: -4px;
		}
	}

	.folder-item {

		&.system .folder-name {
			font-weight: bold;
		}

		.button-delete {
			margin-right: 15px;
			margin-top: 5px;
			visibility: hidden;
			opacity: 0;
		}

		.delete-access {
			&.button-delete {
				visibility: visible;
				margin-right: 0;
				opacity: 1;
			}
		}

		.delete-folder, .subscribe-folder, .unsubscribe-folder, .check-folder, .uncheck-folder {
			cursor: pointer;
			opacity: 0.6;
		}

		.unsubscribe-folder {
			opacity: 0.2;
		}

		.uncheck-folder {
			opacity: 0.2;
		}
	}

	.folder-padding.deep-1 {
		width: 15px * 1 + 10px;
	}
	.folder-padding.deep-2 {
		width: 15px * 2 + 10px;
	}
	.folder-padding.deep-3 {
		width: 15px * 3 + 10px;
	}
	.folder-padding.deep-4 {
		width: 15px * 4 + 10px;
	}
	.folder-padding.deep-5 {
		width: 15px * 5 + 10px;
	}
}