<div class="popups">
	<div class="modal hide b-folder-create-content g-ui-user-select-none" data-bind="modal: modalVisibility">
		<div>
			<div class="modal-header">
				<button type="button" class="close" data-bind="command: cancelCommand">&times;</button>
				<h3>
					<span class="i18n" data-i18n="POPUPS_CREATE_FOLDER/TITLE_CREATE_FOLDER"></span>
				</h3>
			</div>
			<div class="modal-body">
				<div class="form-horizontal">
					<br />
					<div class="control-group">
						<label class="control-label">
							<span class="i18n" data-i18n="POPUPS_CREATE_FOLDER/LABEL_NAME"></span>
						</label>
						<div class="controls">
							<input type="text" class="uiInput inputName" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"
								data-bind="textInput: folderName, hasfocus: folderName.focused, onEnter: createFolderCommand" />
						</div>
					</div>
					<div class="control-group">
						<label class="control-label">
							<span class="i18n" data-i18n="POPUPS_CREATE_FOLDER/LABEL_PARENT"></span>
						</label>
						<div class="controls">
							<select data-bind="options: parentFolderSelectList, value: selectedParentValue,
								optionsText: 'name', optionsValue: 'id', optionsAfterRender: defautOptionsAfterRender"></select>
						</div>
					</div>
				</div>
			</div>
			<div class="modal-footer">
				<a class="btn buttonCreate" data-bind="command: createFolderCommand">
					<i class="icon-folder-add"></i>
					&nbsp;&nbsp;
					<span class="i18n" data-i18n="POPUPS_CREATE_FOLDER/BUTTON_CREATE"></span>
				</a>
			</div>
		</div>
	</div>
</div>
