<div class="e-component e-select inline">
	<!-- ko if: preLabeled -->
	<span class="i18n" data-bind="attr: {'data-i18n': preLabel}"></span>
	&nbsp;&nbsp;
	<!-- /ko -->
	<select data-bind="options: options, value: value, enable: enable, optionsText: optionsText, optionsValue: optionsValue,
		optionsCaption: optionsCaption, css: className, optionsAfterRender: defautOptionsAfterRender"></select>
	<!-- ko if: labeled -->
	&nbsp;
	<span class="i18n" data-bind="attr: {'data-i18n': label}"></span>
	&nbsp;
	<!-- /ko -->
	<!-- ko if: triggered -->
	&nbsp;
	<div data-bind="component: {
		name: 'SaveTrigger',
		params: { value: trigger }
	}"></div>
	<!-- /ko -->
</div>