/*.cke_button__sourcedialog_label {
	display: none !important;
}*/

.cke_chrome {
	border: 1px solid #ccc !important;
}

.cke_toolgroup {
	padding-right: 0 !important;
	background: #FBFBFB !important;
}

.cke_toolgroup, .cke_combo_button {
	border: 1px solid #A6A6A6 !important;
}

.cke_top {
	padding: 6px 4px 1px 6px !important;
	-moz-box-shadow: none !important;
	-webkit-box-shadow: none !important;
	box-shadow: none !important;
	border-bottom: 1px solid #b6b6b6 !important;
	background: #F0F0F0 !important;
}

.cke_combo_on a.cke_combo_button,
.cke_combo_off a.cke_combo_button:hover,
.cke_combo_off a.cke_combo_button:focus,
.cke_combo_off a.cke_combo_button:active {
	padding: 1px !important;
	margin-left: 0 !important;
}

.cke_toolgroup a.cke_button:last-child:after,
.cke_toolgroup a.cke_button.cke_button_disabled:hover:last-child:after {
	border: none !important;
	height: 0 !important;
	width: 0 !important;
	margin: 0 !important;
	padding: 0 !important;
}

.cke_button_on {
	background: #ddd !important;
}

.cke_combo {
	margin-top: 1px !important;
}

.cke_combo__fontsize {
	margin-left: 3px !important;
	.cke_combo_text {
		width: 40px !important;
	}
}

.cke_combo_text {
	line-height: 24px !important;
	padding-left: 8px !important;
}

.cke_combo_button {
	background: #FBFBFB !important;
}

.cke_source {
	font-family: Monaco, Menlo, Consolas, 'Courier New', monospace !important;
	padding: 10px !important;
	padding-right: 0 !important;
	box-shadow: none !important;
}

.cke_plain {

	background-color: #fff;
	outline: none;
	overflow: auto;
	z-index: 1;
	margin: 0;
	border: 0;

	padding: 10px;
	padding-right: 0;
	white-space: pre-wrap;
	font-family: Monaco, Menlo, Consolas, 'Courier New', monospace;
	font-size: 13px;
	line-height: 16px;
	color: #333;
	resize: none;

	border-radius: 0;
	box-shadow: none;
	box-sizing: border-box;
}

.cke_plain:focus, .cke_source:focus {
	border: 0;
}

.b-compose .cke_chrome {
	border-right: 0;
	border-left: 0;
	border-bottom: 0;
	box-shadow: none;
}

.cke_wysiwyg_div {

	padding: 10px !important;
	font-family: arial, sans-serif;
	font-size: 13px;
	line-height: 16px;
	color: #333;

	ul {
		padding-left: 40px;
		li {
			list-style-type: disc !important;
		}
	}

	ol {
		padding-left: 40px;
		li {
			list-style-type: decimal !important;
		}
	}

	pre, code {
		margin: 0px;
		padding: 0px;
		background: #fff;
		border: none;
		border-radius: 0;
		font-family: Monaco, Menlo, Consolas, 'Courier New', monospace;
		display: block;
		word-break: normal;
		word-wrap: break-word;
		background-color: #f9f9f9;
	}

	code {
		display: inline;
		padding: 2px 5px;
	}

	pre {
		padding: 5px 10px;
		border-radius: 5px;
		background-color: #f9f9f9;
	}

	pre > code {
		padding: 0px;
	}

	blockquote {
		border: 0;
		border-left: solid 2px #444;
		margin: 5px 0 5px 5px;
		padding-left: 5px;
	}

	blockquote p {
		margin: 0 0 10px;
		font-size: 14px;
		line-height: 20px;
	}

	img {
		vertical-align: bottom;
	}

	a {
		color: blue;
		text-decoration: underline;

		&:visited {
			color: #609;
		}
		&:active {
			color: red;
		}
	}
}

.cke_dialog {

	a:hover {
		text-decoration: none;
	}

	.cke_dialog_ui_labeled_content {
		margin-top: 5px;
		margin-bottom: 5px;
	}

	.cke_dialog_ui_input_select, .cke_dialog_ui_input_text, .cke_dialog_ui_input_textarea {

		box-shadow: none;
		border-radius: 2px;

		&:focus {
			outline: 0;
			border: 1px solid #999;
		}
	}

	.cke_dialog_ui_input_select, .cke_dialog_ui_input_text {
		height: 25px;
		line-height: 25px;
	}
}
