<div class="b-admin-plugins g-ui-user-select-none">

	<div class="row">
		<div class="alert span8" style="margin-top: 10px;" data-bind="visible: '' !== pluginsError()">
			<button type="button" class="close" data-bind="click: function () { pluginsError('') }">&times;</button>
			<span data-bind="text: pluginsError"></span>
		</div>
	</div>

	<div class="legend" data-i18n="TAB_PLUGINS/LEGEND_PLUGINS"></div>

	<div>
		<div data-bind="component: {
			name: 'Checkbox',
			params: { value: enabledPlugins, label: 'TAB_PLUGINS/LABEL_ENABLE_PLUGINS' }
		}"></div>
	</div>
	<br />
	<div class="row">
		<div class="alert alert-info span8" data-bind="visible: 0 === plugins().length">
			<span data-i18n="TAB_PLUGINS/ALERT_NO_PLUGINS"></span>
			<br />
			<br />
			<strong><a href="#/packages" data-i18n="TAB_PLUGINS/LINK_INSTALL_NEW"></a></strong>
		</div>
		<div class="span8"  data-bind="visible: 0 < plugins().length">
			<blockquote><p class="muted" data-i18n="TAB_PLUGINS/HINT_CLICK_NAME"></p></blockquote>
		</div>
	</div>
	<div class="row">
		<div class="span8">
			<table class="table table-hover b-admin-plugins-list-table" data-bind="i18nUpdate: plugins, visible: 0 < plugins().length, css: {'disabled': !enabledPlugins()}">
				<colgroup>
					<col />
					<col style="width: 30px" />
				</colgroup>
				<tbody data-bind="template: { name: 'AdminSettingsPluginListItem', foreach: plugins }"></tbody>
			</table>
		</div>
	</div>
</div>