<div class="control-group" data-bind="css: {'error': actionValue.error}">
	<div class="controls">
		<input type="text" class="span3 i18n" data-bind="value: actionValue"
			data-i18n="[placeholder]POPUPS_FILTER/EMAIL_LABEL" />
	</div>
</div>
<div class="control-group">
	<div class="controls">
		<div data-bind="component: {
			name: 'Checkbox',
			params: {
				label: 'POPUPS_FILTER/KEEP_LABEL',
				value: actionKeep
			}
		}"></div>
		<div data-bind="visible: $root.allowMarkAsRead, component: {
			name: 'Checkbox',
			params: {
				label: 'POPUPS_FILTER/MARK_AS_READ_LABEL',
				value: actionMarkAsRead
			}
		}"></div>
	</div>
</div>