
#rl-bg {
	z-index: -1;
	position: absolute;
	left: 0;
	right: 0;
	top: 0;
	bottom: 0;
}

#rl-content {
	height: 100%;
	width: 100%;
}

#rl-center {
	.g-ui-absolute-reset;
	min-width: 600px;
	min-height: 400px;
}

html.rl-mobile {

	#rl-center {
		min-width: 250px;
		min-height: 250px;
	}

	&.rl-left-panel-enabled #rl-right {
		right: -150px;
	}

	&.rl-left-panel-disabled #rl-right {
		left: 5px !important;
	}
}

#rl-top {
	.g-ui-absolute-reset;
	bottom: auto;
	z-index: 2;
}

#rl-bottom {
	.g-ui-absolute-reset;

	top: auto;
	z-index: 1;
}

#rl-left {
	.g-ui-absolute-reset;

	width: @rlLeftWidth;
	min-width: 60px;
}

#rl-right {
	.g-ui-absolute-reset;

	z-index: 1;
	left: @rlLeftWidth;
}

#rl-sub-left {
	position: absolute;
	top: 0;
	bottom: 0;
	left: 0;
	width: 500px;
}

#rl-sub-right {
	position: absolute;
	top: 0;
	bottom: 0;
	right: 0;
	left: 500px;

	.b-message-view-backdrop {
		position: absolute;
		top: 0;
		left: 0;
		right: 0;
		bottom: 0;
		background: rgba(0, 0, 0, .4);
		z-index: 10;

		.backdrop-message-wrp {

			position: absolute;
			top: calc(~'50% - 50px');
			left: 0;
			right: 0;
			text-align: center;

			.backdrop-message {
				display: inline-block;
				color: white;
				font-size: 20px;
				line-height: 30px;
				background: rgba(0, 0, 0, .6);
				padding: 15px;
				border-radius: 5px;
				text-shadow: 0px 1px 1px #000;
			}
		}
	}
}

html.ssm-state-desktop-large {

	#rl-left {
		width: @rlLeftWidth + 20;
	}

	#rl-right {
		left: @rlLeftWidth + 20;
	}

	#rl-sub-left {
		width: 500px;
	}

	#rl-sub-right {
		left: 500px;
	}

	.b-compose.modal {
		width: 1000px;
	}

	.b-contacts-content.modal {
		width: 900px;
	}
}

html.ssm-state-desktop {

	#rl-left {
		width: @rlLeftWidth;
	}

	#rl-right {
		left: @rlLeftWidth;
	}

	#rl-sub-left {
		width: 400px;
	}

	#rl-sub-right {
		left: 400px;
	}

	.b-compose.modal {
		width: 1000px;
	}

	.b-contacts-content.modal {
		width: 900px;
	}
}

html.ssm-state-tablet, html.ssm-state-mobile {

	#rl-left {
		width: 155px;
	}

	#rl-right {
		left: 155px;
	}

	#rl-sub-left {
		width: 310px;

		.messageList .inputSearch {
			width: 200px;
		}
	}

	#rl-sub-right {
		left: 310px;
	}

	.b-compose.modal {
		width: 720px;
	}

	.b-contacts-content.modal {
		width: 700px;
	}

	.b-contacts-content.modal {
		.contactValueInput {
			width: 200px;
		}
	}
}

html.ssm-state-tablet {

	.b-compose.modal {
		width: 720px;
	}

	.b-contacts-content.modal {
		width: 800px;
	}

	.b-contacts-content.modal {
		.contactValueInput {
			width: 250px;
		}
	}
}

.show-on-panel-disabled {
	display: none;
}

html.rl-left-panel-disabled {

	#rl-left {
		width: 60px !important;

		.show-on-panel-disabled {
			display: block;
		}

		.opacity-on-panel-disabled {
			opacity: 0.3;
		}

		.visibility-hidden-on-panel-disabled {
			visibility: hidden;
		}

		.hide-on-panel-disabled {
			display: none;
		}

		&.ui-state-disabled {
			opacity: 1;
		}
	}

	#rl-right {
		left: 60px !important;
	}
}

html.rl-left-panel-short {

	#rl-left {
		width: 60px !important;
	}

	#rl-right {
		left: 60px !important;
	}
}

html.rl-left-panel-none {

	#rl-left {
		width: 10px !important;
	}

	#rl-right {
		left: 10px !important;
	}
}

.ui-resizable-helper-w {
	border-right: 5px solid #777;
	border-right-color: rgba(255,255,255,0.7);
}

.ui-resizable-helper-h {
	border-bottom: 5px solid #ccc;
	border-bottom-color: rgba(0,0,0,0.3);
}

.ui-resizable-handle:hover {
	background: #aaa;
	background: rgba(255,255,255,0.5);
}

html.rl-no-preview-pane {

	#rl-sub-left {
		right: @rlBottomMargin !important;
		width: inherit;

		.messageList .inputSearch {
			width: 300px !important;
		}
	}

	#rl-sub-right {
		left: 0 !important;
	}

	#rl-right .ui-resizable-handle {
		display: none !important;
	}
}

html.rl-side-preview-pane #rl-right .ui-resizable-handle {
	display: none !important;
}

html.rl-bottom-preview-pane {

	#rl-sub-left {
		right: @rlBottomMargin !important;
		width: inherit;

		.b-message-list-wrapper {
			bottom: inherit;
			height: 300px;

			box-shadow: none;
		}

		.messageList .inputSearch {
			width: 300px !important;
		}
	}

	#rl-sub-right {

		left: 0 !important;

		.messageView .top-toolbar {
			display: none;
		}

		.b-message-view-wrapper {
			top: 356px;
			left: 0;
			right: 5px;
			box-shadow: none;
		}
	}
}

.visible-on-ctrl, .visible-on-ctrl-btn {
	display: none;
}

.hidden-on-ctrl-btn {
	display: inline-block;
}

html.rl-ctrl-key-pressed {

	.visible-on-ctrl {
		display: inline-block;
	}

	.btn-group.open .visible-on-ctrl-btn {
		display: inline-block;
	}

	.hidden-on-ctrl {
		display: none;
	}

	.btn-group.open .hidden-on-ctrl-btn {
		display: none;
	}
}

#rl-loading, #rl-loading-error {
	position: absolute;
	font-size: 30px;
	line-height: 130%;
	top: 50%;
	width: 100%;
	height: 65px;
	margin: 0;
	margin-top: -60px;
	background-color: transparent;
	text-align: center;
	color: #000;
}

#rl-loading-error {
	background-image: none;
	display: none;
}

#rl-loading-error-additional {
	margin-top: 20px;
	font-size: 20px;
}

#rl-app{
	display: none;
}

#rl-check{
	display: none;
}

.rl-content-show {
	display: block !important;
}

html.rl-mobile .hide-on-mobile {
	display: none !important;
}

.show-on-mobile {
	display: none !important;
}

html.rl-mobile .show-on-mobile {
	display: initial !important;
}

html.rl-mobile .width100-on-mobile {
	width: 100% !important;
}
