
.b-settings-accounts {

	html.rl-mobile &{
		margin-right: 15px;
	}

	.process-place {
		text-align: center;
		width: 600px;
		padding: 14px 0;
	}

	.list-table {

		width: 600px;

		td {
			padding: 4px 8px;
			line-height: 30px;
		}

		.drag-handle {
			color: #eee;
		}

		tr:hover .drag-handle {
			color: #aaa;
		}

		.account-img, .identity-img {
			font-size: 12px;
			margin-right: 5px;
		}

		.account-name {
			display: inline-block;
			word-break: break-all;
			box-sizing: border-box;
			line-height: 22px;
		}

		.identity-default {
			cursor: pointer;
			color: #ccc;
		}

		.identity-name {
			display: inline-block;
			word-break: break-all;
			box-sizing: border-box;
			line-height: 22px;
			cursor: pointer;
		}
	}

	.accounts-list-top-padding, .identities-list-top-padding {
		display: inline-block;
		height: 5px;
		width: 5px;
	}

	.account-item {

		white-space: nowrap;

		.e-action {
			cursor: pointer;
		}

		.drag-handle {
			cursor: pointer;
			cursor: all-scroll;
		}

		.button-delete {
			margin-right: 15px;
			margin-top: 5px;
			visibility: hidden;
			opacity: 0;
		}

		.delete-access {
			&.button-delete {
				visibility: visible;
				margin-right: 0;
				opacity: 1;
			}
		}

		.delete-account {
			cursor: pointer;
			opacity: 0.5;
		}

		&.ui-sortable-helper {
			background: #F5F5F5;
			border: 1px solid #DDD;

			.button-delete, .delete-account {
				display: none;
			}

			td {
				border-width: 0px;
			}
		}
	}

	.identity-item {

		white-space: nowrap;

		.e-action {
			cursor: pointer;
		}

		.button-delete {
			margin-right: 15px;
			margin-top: 5px;
			visibility: hidden;
			opacity: 0;
		}

		.delete-access {
			&.button-delete {
				visibility: visible;
				margin-right: 0;
				opacity: 1;
			}
		}

		.delete-identity {
			cursor: pointer;
			opacity: 0.5;
		}

		&.ui-sortable-helper {
			background: #F5F5F5;
			border: 1px solid #DDD;

			.button-delete, .delete-identity {
				display: none;
			}

			td {
				border-width: 0px;
			}
		}
	}
}