<span class="e-component e-checkbox inline" tabindex="0" data-bind="click: click, onSpace: click, css: { 'disabled': disable() || !enable() }">
	<i role="checkbox" class="e-checkbox-icon" data-bind="css: value() ?
	   (inverted ? 'icon-checkbox-unchecked' : 'icon-checkbox-checked') :
	   (inverted ? 'icon-checkbox-checked' : 'icon-checkbox-unchecked')
	"></i>
<!-- ko if: labeled -->
	&nbsp;&nbsp;
	<span class="sub-label i18n" data-bind="attr: {'data-i18n': label}, css: {'i18n-animation': labelAnimated}"></span>
<!-- /ko -->
</span>