
.b-admin-plugins {
	.process-place {
		visibility: hidden;
	}
}

.b-admin-plugins-list-table {

	&.disabled {
		opacity: 0.5;
		background-color: #eee;
	}

	.e-item {
		.e-action {
			cursor: pointer;
		}

		.plugin-img {
			font-size: 12px;
			margin-right: 2px;
		}

		&.disabled {
			.plugin-img, .plugin-name {
				color: #bbb;
			}
			.disabled-plugin {
				opacity: 0.5;
			}
		}
	}
}

.b-admin-plugin-property {
	.help-block {
		margin-bottom: 5px;
	}
}
