<tr class="folder-item" data-bind="css: { 'selectable': selectable, 'system': isSystemFolder }">
	<td>
		<span class="folder-padding" data-bind="css: 'deep-' + deep">&nbsp;</span>
		<span class="folder-name" data-bind="text: name, visible: !edited(), css: { 'e-action': canBeEdited, 'can-be-edited': canBeEdited }, click: function (oFolder) { $root.folderForEdit(oFolder); }"></span>
		&nbsp;
		<span class="folder-system-name" data-bind="text: manageFolderSystemName, visible: isSystemFolder"></span>
		<input type="text" class="folder-name-input"
			autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"
			data-bind="value: nameForEdit, visible: edited, hasfocus: edited, onEnter: function () { $root.folderEditOnEnter($data); }, onEsc: function () { $root.folderEditOnEsc($data); }" />

		<a class="btn btn-small btn-small-small btn-danger pull-right button-delete button-delete-transitions" data-bind="css: {'delete-access': deleteAccess()}, click: function() { $root.deleteFolder($data); }">
			<span class="i18n" data-i18n="SETTINGS_FOLDERS/DELETING_ASK"></span>
		</a>
	</td>
	<td class="delete-folder-parent">
		<span class="delete-folder" data-bind="visible: canBeDeleted() && !deleteAccess(), click: function (oFolder) { $root.folderForDeletion(oFolder); }">
			<i class="icon-trash"></i>
		</span>
	</td>
	<td class="subscribe-folder-parent">
		<span class="unsubscribe-folder" data-bind="visible: canBeSubScribed() && !subScribed(), click: function(oFolder) { $root.subscribeFolder(oFolder); }">
			<i class="icon-eye"></i>
		</span>
		<span class="subscribe-folder" data-bind="visible: canBeSubScribed() && subScribed(), click: function(oFolder) { $root.unSubscribeFolder(oFolder); }">
			<i class="icon-eye"></i>
		</span>
	</td>
	<td class="check-folder-parent" data-bind="visible: $root.displaySpecSetting">
		<span class="uncheck-folder" data-bind="visible: canBeChecked() && subScribed() && !checkable(), click: function(oFolder) { $root.checkableTrueFolder(oFolder); }">
			<i class="icon-check-mark-circle-two"></i>
		</span>
		<span class="check-folder" data-bind="visible: canBeChecked() && subScribed() && checkable(), click: function(oFolder) { $root.checkableFalseFolder(oFolder); }">
			<i class="icon-check-mark-circle-two"></i>
		</span>
	</td>
</tr>
<!-- ko template: { name: 'SettingsFolderItem', foreach: subFolders } --><!-- /ko -->