<div style="margin: 30px">
	<h1 style="display: inline; color: #fff; text-shadow: 1px 2px 0px rgba(0, 0, 0, 0.5);">RainLoop Webmail</h1>
	&nbsp;&nbsp;&nbsp;
	<h3 style="display: inline; color: #fff; text-shadow: 1px 2px 0px rgba(0, 0, 0, 0.5);">(<span data-bind="text: version"></span>)</h3>
	<h3><a href="http://rainloop.net/" target="_blank">http://rainloop.net</a></h3>
</div>