
.inputosaurus-container {

	width: 99%;
	line-height: 20px;
	padding: 2px;
	border: 1px solid #cccccc;
	border-radius: 3px;

	box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);
	.transition(~"border linear .2s, box-shadow linear .2s");

	&.inputosaurus-focused {
		background-color: #fff;
		border: @rlInputBorderSize solid darken(@inputBorder, 20%);
		box-shadow: none;
	}

	li {
		max-width: 500px;
		background-color: #eee;
		border: 1px solid #aaa;
		line-height: 18px;
		padding: 2px 15px 2px 5px;
		margin: 2px 2px 2px;
		color: #555;
		z-index: 100;

		&.ui-draggable.ui-draggable-dragging {
			z-index: 101;
		}

		position: relative;

		a {
			color: #999;
			font-size: 12px;
			position: absolute;
			top: 1px;
			right: 2px;

			&:hover {
				color: #666;
			}
		}

		span {
			padding-right: 3px;
		}

		&.inputosaurus-required {
			padding-right: 5px;
		}

		&.inputosaurus-selected {
			background-color: #ddd;
		}

		&.pgp {
			background-color: #E5F3E2;
		}
	}

	.inputosaurus-input {

		margin: 1px 10px 1px 0px;
		height: 22px;

		input {
			border: 0px;
			height : 21px;
			padding-left: 0;
		}
	}
}

.ui-autocomplete {
	z-index: 2000;
}

.inputosaurus-fake-span {
	position: absolute;
	top: 0;
	left: -5000px;
}
