
import {componentExportHelper} from 'Component/Abstract';
import {AbstractInput} from 'Component/AbstractInput';

class DateComponent extends AbstractInput {}

export default componentExportHelper(DateComponent, 'DateComponent');
