<div class="popups">
	<div class="modal hide b-two-factor-test-content g-ui-user-select-none" data-bind="modal: modalVisibility">
		<div>
			<div class="modal-header">
				<button type="button" class="close" data-bind="command: cancelCommand">&times;</button>
				<h3>
					<span class="i18n" data-i18n="POPUPS_TWO_FACTOR_TEST/TITLE_TEST_CODE"></span>
				</h3>
			</div>
			<div class="modal-body">
				<div class="form-horizontal">
					<br />
					<div class="control-group">
						<label class="control-label">
							<span class="i18n" data-i18n="POPUPS_TWO_FACTOR_TEST/LABEL_CODE"></span>
						</label>
						<div class="controls">
							<input type="text" class="uiInput inputName"
								autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"
								data-bind="textInput: code, hasfocus: code.focused, onEnter: testCodeCommand" />
						</div>
					</div>
				</div>
			</div>
			<div class="modal-footer">
				<a class="btn" data-bind="command: testCodeCommand, css: { 'btn-success': true === code.status(), 'btn-danger': false === code.status() }">
					<i data-bind="css: {'icon-ok': !testing(), 'icon-spinner animated': testing(), 'icon-white': true === code.status() || false === code.status() }"></i>
					&nbsp;&nbsp;
					<span class="i18n" data-i18n="POPUPS_TWO_FACTOR_TEST/BUTTON_TEST"></span>
				</a>
			</div>
		</div>
	</div>
</div>
