<div class="e-pagenator g-ui-user-select-none" data-bind="foreach: $data">
	<a class="e-page" data-bind="css: { 'current': current, 'custom': custom }, title: title">
		<span class="e-page-number" data-bind="text: name"></span>
	</a>
</div>