<div class="b-settins-left g-ui-user-select-none">
	<div class="b-content thm-settings-menu opacity-on-panel-disabled">
		<div class="b-settings-menu" data-bind="foreach: menu">
			<!-- ko if: disabled -->
			<span class="e-item disabled">
				<span class="i18n e-link" data-bind="attr: { 'data-i18n': label }"></span>
			</span>
			<!-- /ko -->
			<!-- ko ifnot: disabled -->
			<a class="e-item selectable" href="#" data-bind="css: {'selected': selected }, attr: { 'href': $root.link(route) }">
				<span class="i18n e-link" data-bind="attr: { 'data-i18n': label }"></span>
			</a>
			<!-- /ko -->
		</div>
	</div>
	<div class="b-content show-on-panel-disabled" data-bind="click: function () { leftPanelDisabled(false); }"></div>
	<div class="b-footer">
		<nobr>
			<div class="btn-toolbar">
				<div class="btn-group">
					<a class="btn single buttonResize" data-bind="click: function () { leftPanelDisabled(!leftPanelDisabled()); }">
						<i data-bind="css: {'icon-resize-out': leftPanelDisabled(), 'icon-resize-in': !leftPanelDisabled()}"></i>
					</a>
				</div>
			</div>
		</nobr>
	</div>
</div>