
.nano.scroller-shadow-top:before, .nano.scroller-shadow-bottom:after {
	content: "";
	position: absolute;
	left: 0;
	width: 100%;
	height: 10px;
	z-index: 102;
	box-shadow: 0px 0px 20px rgba(0, 0, 0, 0.4);
}

.nano.scroller-shadow-top:before {
	top: -10px;
}

.nano.scroller-shadow-bottom:after {
	bottom: -10px;
}

.nano.fixIndex.scroller-shadow-top:before, .nano.fixIndex.scroller-shadow-bottom:after {
	z-index: 98;
}

/*
.nano {
	.pane .slider {
		.transition(top 0.1s ease-out);
	}

	.pane2 .slider2 {
		.transition(left 0.1s ease-out);
	}

	.pane.active .slider, .pane2.active .slider2 {
		.transition(none);
	}
}
*/
