<div class="b-admin-general">
	<div class="form-horizontal">
		<div class="legend" data-i18n="TAB_LOGIN/LEGEND_LOGIN_SCREEN"></div>
		<div class="control-group">
			<label class="control-label" data-i18n="TAB_LOGIN/LABEL_DEFAULT_DOMAIN"></label>
			<div class="controls">
				<div data-bind="component: {
					name: 'Input',
					params: {
						value: defaultDomain,
						trigger: defaultDomain.trigger,
						size: 3
					}
				}"></div>
			</div>
		</div>
		<div class="control-group">
			<div class="controls">
				<div data-bind="component: {
					name: 'Checkbox',
					params: { value: determineUserDomain, label: 'TAB_LOGIN/LABEL_DETERMINE_USER_DOMAIN' }
				}"></div>
				<br />
				<div data-bind="component: {
					name: 'Checkbox',
					params: { value: allowLanguagesOnLogin, label: 'TAB_LOGIN/LABEL_ALLOW_LANGUAGES_ON_LOGIN' }
				}"></div>
				<div data-bind="visible: allowLanguagesOnLogin, component: {
					name: 'Checkbox',
					params: { enable: allowLanguagesOnLogin, value: determineUserLanguage, label: 'TAB_LOGIN/LABEL_DETERMINE_USER_LANGUAGE' }
				}"></div>
			</div>
		</div>
	</div>
</div>