
.b-admin-packages {

	.alert {
		width: 650px;
	}

	.process-place {
		visibility: hidden;
	}
}

.b-admin-packages-list-table {

	width: 700px;

	.e-item {

		.package-img {
			font-size: 12px;
			margin-right: 2px;
		}

		.package-name.core {
			font-weight: bold;
		}
		.package-desc {
			color: #999;
			font-size: 12px;
		}

		.e-action {
			cursor: pointer;
		}

		.package-release-parent, .package-actions-parent {
			text-align: center;
		}
		.package-actions-parent {
			vertical-align: middle;
		}
	}
}
