<div class="popups">
	<div class="modal hide b-open-pgp-key-add-content g-ui-user-select-none" data-bind="modal: modalVisibility">
		<div>
			<div class="modal-header">
				<button type="button" class="close" data-bind="command: cancelCommand">&times;</button>
				<h3>
					<span class="i18n" data-i18n="POPUPS_IMPORT_OPEN_PGP_KEY/TITLE_IMPORT_OPEN_PGP_KEY"></span>
				</h3>
			</div>
			<div class="modal-body">
				<div class="alert"  data-bind="visible: key.error() && key.errorMessage(), text: key.errorMessage"></div>
				<div class="form-horizontal">
					<div class="control-group" data-bind="css: {'error': key.error}">
						<textarea class="inputKey input-xxlarge" rows="14" autocomplete="off" data-bind="value: key, hasfocus: key.focus"></textarea>
					</div>
				</div>
			</div>
			<div class="modal-footer">
				<a class="btn buttonAddAccount" data-bind="command: addOpenPgpKeyCommand">
					<i class="icon-list-add"></i>
					&nbsp;&nbsp;
					<span class="i18n" data-i18n="POPUPS_IMPORT_OPEN_PGP_KEY/BUTTON_IMPORT_OPEN_PGP_KEY"></span>
				</a>
			</div>
		</div>
	</div>
</div>
