<div class="popups">
	<div class="modal hide b-message-open-pgp-content g-ui-user-select-none" data-bind="modal: modalVisibility">
		<div>
			<div class="modal-header">
				<button type="button" class="close" data-bind="command: cancelCommand">&times;</button>
				<h3>
					<span class="i18n" data-i18n="POPUPS_MESSAGE_OPEN_PGP/TITLE_MESSAGE_OPEN_PGP"></span>
				</h3>
			</div>
			<div class="modal-body">
				<div class="form-horizontal">
					<div class="alert" data-bind="visible: '' !== notification()">
						<span data-bind="text: notification"></span>
					</div>
					<br />
					<div class="control-group">
						<label class="i18n control-label" data-i18n="POPUPS_MESSAGE_OPEN_PGP/LABEL_KEY"></label>
						<div class="controls key-list" data-bind="foreach: privateKeys" style="margin-top: 5px">
							<div class="key-list__item">
								<i class="key-list__item__radio icon-radio-unchecked"></i>
								<div class="key-list__item__names">
									<span data-bind="foreach: users">
										<div>
										<span class="key-list__item__name">
											<span data-bind="text: $data"></span>
											&nbsp;
											<span>[<span data-bind="text: $parent.id"></span>]</span>
										</span>
										</div>
									</span>
								</div>
							</div>
							<!--<div data-bind="json: $data"></div>-->
						</div>
					</div>
					<div class="control-group">
						<label class="i18n control-label" data-i18n="POPUPS_MESSAGE_OPEN_PGP/LABEL_PASSWORD"></label>
						<div class="controls">
							<input type="password" class="inputPassword input-xlarge" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"
								data-bind="textInput: password, hasfocus: password.focus, onEnter: doCommand" />
						</div>
					</div>
				</div>
			</div>
			<div class="modal-footer">
				<button class="btn buttonDo" data-bind="command: doCommand, hasfocus: buttonFocus">
					<i data-bind="css: {'icon-key': !submitRequest(), 'icon-spinner animated': submitRequest()}"></i>
					&nbsp;&nbsp;
					<span class="i18n" data-i18n="POPUPS_MESSAGE_OPEN_PGP/BUTTON_DECRYPT"></span>
				</button>
			</div>
		</div>
	</div>
</div>
