
.progressjs-theme-rainloop {
	z-index: 2000;
}

.progressjs-theme-rainloop .progressjs-inner {

	background-color: #939595;
	position: relative;
	z-index: 2000;
	height: 3px;
	overflow: hidden;

	-webkit-transition: width .5s;
	-moz-transition: width .5s;
	-o-transition: width .5s;
	transition: width .5s;
}

.progressjs-theme-rainloop .progressjs-percent {
	position: absolute;
	top: 0;
	left: 0;
	right: -32px;
	bottom: 0;

	background-image: -webkit-gradient(linear, 0 100%, 100% 0, color-stop(0.25, rgba(255, 255, 255, 0.3)), color-stop(0.25, transparent), color-stop(0.5, transparent), color-stop(0.5, rgba(255, 255, 255, 0.3)), color-stop(0.75, rgba(255, 255, 255, 0.3)), color-stop(0.75, transparent), to(transparent));
	background-image: -webkit-linear-gradient(45deg, rgba(255, 255, 255, 0.3) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.3) 50%, rgba(255, 255, 255, 0.3) 75%, transparent 75%, transparent);
	background-image: -moz-linear-gradient(45deg, rgba(255, 255, 255, 0.3) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.3) 50%, rgba(255, 255, 255, 0.3) 75%, transparent 75%, transparent);
	background-image: -o-linear-gradient(45deg, rgba(255, 255, 255, 0.3) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.3) 50%, rgba(255, 255, 255, 0.3) 75%, transparent 75%, transparent);
	background-image: linear-gradient(45deg, rgba(255, 255, 255, 0.3) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.3) 50%, rgba(255, 255, 255, 0.3) 75%, transparent 75%, transparent);
	-webkit-background-size: 32px 32px;
	-moz-background-size: 32px 32px;
	-o-background-size: 32px 32px;
	background-size: 32px 32px;

	-webkit-animation: simple-pace-stripe-animation 500ms linear infinite;
	-moz-animation: simple-pace-stripe-animation 500ms linear infinite;
	-ms-animation: simple-pace-stripe-animation 500ms linear infinite;
	-o-animation: simple-pace-stripe-animation 500ms linear infinite;
	animation: simple-pace-stripe-animation 500ms linear infinite;
}

@-webkit-keyframes simple-pace-stripe-animation {
	0% { -webkit-transform: none; transform: none; }
	100% { -webkit-transform: translate(-32px, 0); transform: translate(-32px, 0); }
}
@-moz-keyframes simple-pace-stripe-animation {
	0% { -moz-transform: none; transform: none; }
	100% { -moz-transform: translate(-32px, 0); transform: translate(-32px, 0); }
}
@-o-keyframes simple-pace-stripe-animation {
	0% { -o-transform: none; transform: none; }
	100% { -o-transform: translate(-32px, 0); transform: translate(-32px, 0); }
}
@-ms-keyframes simple-pace-stripe-animation {
	0% { -ms-transform: none; transform: none; }
	100% { -ms-transform: translate(-32px, 0); transform: translate(-32px, 0); }
}
@keyframes simple-pace-stripe-animation {
	0% { transform: none; transform: none; }
	100% { transform: translate(-32px, 0); transform: translate(-32px, 0); }
}
