
// MAIN
@main-color: #333;
@main-background-color: #eee;
@main-background-image: "images/background.jpg";

// LOADING
@loading-color: #000;

// LOGIN
@login-color: #eee;
@login-background-color: #757575;
@login-rgba-background-color: rgba(0,0,0,0.5);
@login-box-shadow: none;
@login-border: none;
@login-border-radius: 7px;

@spinner-background: rgba(0,0,0,.2);

// MENU
@dropdown-menu-color: #333;
@dropdown-menu-background-color: #fff;
@dropdown-menu-hover-background-color: #757575;
@dropdown-menu-hover-color: #eee;
@dropdown-menu-disable-color: #999;

// FOLDERS
@folders-color: #333;
@folders-disabled-color: #999;
@folders-selected-color: #eee;
@folders-selected-background-color: #757575;
@folders-selected-rgba-background-color: rgba(0,0,0,0.5);
@folders-focused-color: #eee;
@folders-focused-background-color: #757575;
@folders-focused-rgba-background-color: rgba(0,0,0,0.7);
@folders-hover-color: #eee;
@folders-hover-background-color: #757575;
@folders-hover-rgba-background-color: rgba(0,0,0,0.5);
@folders-drop-color: #eee;
@folders-drop-background-color: #757575;
@folders-drop-rgba-background-color: rgba(0,0,0,0.5);

// SETTINGS
@settings-menu-color: #333;
@settings-menu-disabled-color: #999;
@settings-menu-selected-color: #eee;
@settings-menu-selected-background-color: #757575;
@settings-menu-selected-rgba-background-color: rgba(0,0,0,0.5);
@settings-menu-hover-color: #eee;
@settings-menu-hover-background-color: #757575;
@settings-menu-hover-rgba-background-color: rgba(0,0,0,0.5);

// MESSAGE LIST
@message-list-toolbar-background-color: #eee;
@message-list-toolbar-gradient-start: #f4f4f4;
@message-list-toolbar-gradient-end: #dfdfdf;
