<div class="control-group" data-bind="css: {'error': actionValue.error}">
	<div class="controls">
		<select class="span3" data-bind="options: $root.folderSelectList, value: $root.selectedFolderValue,
			optionsText: 'name', optionsValue: 'id', optionsAfterRender: $root.defautOptionsAfterRender"></select>
	</div>
</div>
<div class="control-group">
	<div class="controls">
		<div data-bind="visible: $root.allowMarkAsRead, component: {
			name: 'Checkbox',
			params: {
				label: 'POPUPS_FILTER/MARK_AS_READ_LABEL',
				value: actionMarkAsRead
			}
		}"></div>
	</div>
</div>