<div class="pswp" tabindex="-1" role="dialog" aria-hidden="true">
    <div class="pswp__bg"></div>
    <div class="pswp__scroll-wrap">
        <div class="pswp__container">
            <div class="pswp__item"></div>
            <div class="pswp__item"></div>
            <div class="pswp__item"></div>
        </div>
        <div class="pswp__ui pswp__ui--hidden">
            <div class="pswp__top-bar">
                <div class="pswp__counter"></div>
                <button class="pswp__button pswp__button--close i18n" title="Close (Esc)" data-i18n="[title]PREVIEW_POPUP/CLOSE"></button>
                <button class="pswp__button pswp__button--fs i18n" title="Toggle fullscreen" data-i18n="[title]PREVIEW_POPUP/FULLSCREEN"></button>
                <button class="pswp__button pswp__button--zoom i18n" title="Zoom in/out" data-i18n="[title]PREVIEW_POPUP/ZOOM"></button>
                <div class="pswp__preloader">
                    <div class="pswp__preloader__icn">
						<div class="pswp__preloader__cut">
							<div class="pswp__preloader__donut"></div>
						</div>
                    </div>
                </div>
            </div>
            <div class="pswp__share-modal pswp__share-modal--hidden pswp__single-tap">
                <div class="pswp__share-tooltip"></div>
            </div>
            <button class="pswp__button pswp__button--arrow--left i18n" title="Previous (arrow left)" data-i18n="[title]PREVIEW_POPUP/GALLERY_PREV"></button>
            <button class="pswp__button pswp__button--arrow--right i18n" title="Next (arrow right)" data-i18n="[title]PREVIEW_POPUP/GALLERY_NEXT"></button>
            <div class="pswp__caption">
                <div class="pswp__caption__center"></div>
            </div>
		</div>
	</div>
</div>