<div class="b-settings-templates g-ui-user-select-none">
	<div class="form-horizontal">
		<div class="legend">
			<span class="i18n" data-i18n="SETTINGS_TEMPLATES/LEGEND_TEMPLATES"></span>
			&nbsp;&nbsp;&nbsp;
			<i class="icon-spinner animated" style="margin-top: 5px" data-bind="visible: templates.loading"></i>
		</div>
	</div>
	<a class="btn" data-bind="click: addNewTemplate">
		<i class="icon-user-add"></i>
		&nbsp;&nbsp;
		<span class="i18n" data-i18n="SETTINGS_TEMPLATES/BUTTON_ADD_TEMPLATE"></span>
	</a>
	<br />
	<br />
	<br />
	<div class="templates-list-wrp">
		<div class="templates-list-top-padding"></div>
			<table class="table table-hover list-table templates-list" data-bind="i18nUpdate: templates">
				<colgroup>
					<col />
					<col style="width: 150px" />
					<col style="width: 1%" />
				</colgroup>
				<tbody data-bind="sortable: {data: templates, options: scrollableOptions('.templates-list-wrp')}">
					<tr class="template-item">
						<td class="e-action">
							<span class="template-img icon-user"></span>
							<i class="icon-braille drag-handle"></i>
							&nbsp;&nbsp;
							<span class="template-name" data-bind="text: name"></span>
						</td>
						<td>
							<a class="btn btn-small btn-small-small btn-danger pull-right button-delete button-delete-transitions" data-bind="css: {'delete-access': deleteAccess}, click: function(oTemplate) { $root.deleteTemplate(oTemplate); }">
								<span class="i18n" data-i18n="SETTINGS_TEMPLATES/DELETING_ASK"></span>
							</a>
						</td>
						<td>
							<span class="delete-template" data-bind="visible: !deleteAccess(), click: function (oTemplate) { $root.templateForDeletion(oTemplate); }">
								<i class="icon-trash"></i>
							</span>
						</td>
					</tr>
				</tbody>
			</table>
		</div>
	</div>
</div>