
/**
 * ownCloud - RainLoop mail plugin
 *
 * @author RainLoop Team
 *
 * https://github.com/RainLoop/rainloop-webmail/tree/master/build/owncloud
 */

$(function() {
	RainLoopFormHelper('#mail-rainloop-admin-form', 'admin.php');
});
