<div class="popups">
	<div class="modal hide b-template-add-content" data-bind="modal: modalVisibility">
		<div>
			<div class="modal-header g-ui-user-select-none">
				<button type="button" class="close" data-bind="command: cancelCommand">&times;</button>
				<h3>
					<span data-bind="visible: '' === id()" class="i18n" data-i18n="POPUPS_ADD_TEMPLATE/TITLE_ADD_TEMPLATE"></span>
					<span data-bind="visible: '' !== id()" class="i18n" data-i18n="POPUPS_ADD_TEMPLATE/TITLE_UPDATE_TEMPLATE"></span>
				</h3>
			</div>
			<div class="modal-body">
				<div class="form-horizontal g-ui-user-select-none">
					<div class="alert" data-bind="visible: '' !== submitError()">
						<button type="button" class="close" data-bind="click: function () { submitError('') }">&times;</button>
						<span data-bind="text: submitError"></span>
					</div>
					<br />
					<div class="control-group" data-bind="css: {'error': name.error}">
						<label class="i18n control-label" data-i18n="POPUPS_ADD_TEMPLATE/LABEL_NAME"></label>
						<div class="controls">
							<input type="text" class="inputName input-xlarge" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"
								data-bind="textInput: name, onEnter: addTemplateCommand, hasfocus: name.focus" />
						</div>
					</div>
				</div>
				<hr />
				<div class="form-horizontal">
					<div class="control-group" data-bind="css: {'error': body.error}">
						<div class="e-template-place" data-bind="initDom: signatureDom"></div>
					</div>
				</div>
			</div>
			<div class="modal-footer">
				<a class="btn buttonAddAccount" data-bind="command: addTemplateCommand">
					<i data-bind="visible: '' == id(), css: {'icon-user-add': !submitRequest(), 'icon-spinner animated': submitRequest()}"></i>
					<i data-bind="visible: '' !== id(), css: {'icon-ok': !submitRequest(), 'icon-spinner animated': submitRequest()}"></i>
					&nbsp;&nbsp;
					<span data-bind="visible: '' == id()" class="i18n" data-i18n="POPUPS_ADD_TEMPLATE/BUTTON_ADD_TEMPLATE"></span>
					<span data-bind="visible: '' !== id()" class="i18n" data-i18n="POPUPS_ADD_TEMPLATE/BUTTON_UPDATE_TEMPLATE"></span>
				</a>
			</div>
		</div>
	</div>
</div>
