<div class="b-admin-plugin-property">
	<label class="control-label">
		<span data-bind="text: Label, visible: 5 !== Type"></span>
	</label>
	<div class="controls">
		<!-- ko if: 0 === Type || 1 === Type -->
			<input type="text" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"
				data-bind="value: value, attr: {placeholder: placeholder}" />
		<!-- /ko -->
		<!-- ko if: 3 === Type -->
			<input type="password" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"
				data-bind="value: value, attr: {placeholder: placeholder}" />
		<!-- /ko -->
		<!-- ko if: 2 === Type -->
			<div data-bind="component: {
				name: 'TextArea',
				params: { value: value, placeholder: placeholder }
			}"></div>
		<!-- /ko -->
		<!-- ko if: 4 === Type -->
			<select data-bind="options: Default, value: value"></select>
		<!-- /ko -->
		<!-- ko if: 5 === Type -->
			<div data-bind="component: {
				name: 'Checkbox',
				params: { value: value, label: Label }
			}"></div>
		<!-- /ko -->
		<span class="help-block" style="color: #ccc" data-bind="text: Desc, visible: '' !== Desc"></span>
	</div>
	<br />
</div>