html {
	margin: 0;
	padding: 0;
	overflow: hidden;
	font-family: Arial, Verdana, Geneva, sans-serif;
	height: 100%;
}

body {
	margin: 0;
	padding: 0;
	overflow: hidden;
	background-color: #e3e3e3;
	font-family: Arial, Verdana, Geneva, sans-serif;

	-webkit-touch-callout: none;

	position: absolute;
	top: 0;
	left: 0;
	bottom: 0;
	right: 0;
	height: 100%;
}

textarea {
	resize: none;
}

option:disabled {
	color: #aaa;
	cursor: not-allowed;
}

body:before {
	content: "";
	position: fixed;
	top: -10px;
	left: 0;
	width: 100%;
	height: 8px;
	z-index: 100;
	box-shadow: 0px 0px 10px rgba(0,0,0,.6);
}

* {
	outline: none;
}

select:focus {
    outline: none;
}

html.mobile *, html.rl-mobile * {
	-webkit-tap-highlight-color: rgba(0,0,0,0);
}

input[type="search"]{
	box-sizing: content-box;
}

input::-ms-clear,
input::-ms-reveal {
	display: none;
}