<?php

namespace RainLoop\Enumerations;

class Capa
{
	const PREM = 'PREM';
	const TWO_FACTOR = 'TWO_FACTOR';
	const TWO_FACTOR_FORCE = 'TWO_FACTOR_FORCE';
	const OPEN_PGP = 'OPEN_PGP';
	const PREFETCH = 'PREFETCH';
	const GRAVATAR = 'GRAVATAR';
	const THEMES = 'THEMES';
	const USER_BACKGROUND = 'USER_BACKGROUND';
	const SIEVE = 'SIEVE';
	const FILTERS = 'FILTERS';
	const ATTACHMENT_THUMBNAILS = 'ATTACHMENT_THUMBNAILS';
	const ADDITIONAL_ACCOUNTS = 'ADDITIONAL_ACCOUNTS';
	const IDENTITIES = 'IDENTITIES';
	const FOLDERS = 'FOLDERS';
	const COMPOSER = 'COMPOSER';
	const CONTACTS = 'CONTACTS';
	const RELOAD = 'RELOAD';
	const SEARCH = 'SEARCH';
	const SEARCH_ADV = 'SEARCH_ADV';
	const SETTINGS = 'SETTINGS';
	const QUOTA = 'QUOTA';
	const HELP = 'HELP';
	const TEMPLATES = 'TEMPLATES';
	const MESSAGE_ACTIONS = 'MESSAGE_ACTIONS';
	const MESSAGELIST_ACTIONS = 'MESSAGELIST_ACTIONS';
	const ATTACHMENTS_ACTIONS = 'ATTACHMENTS_ACTIONS';
	const DANGEROUS_ACTIONS = 'DANGEROUS_ACTIONS';
	const AUTOLOGOUT = 'AUTOLOGOUT';
}
