<?php

/*
 * This file is part of MailSo.
 *
 * (c) 2014 Usenko Timur
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace MailSo\Base\Enumerations;

/**
 * @category MailSo
 * @package Base
 * @subpackage Enumerations
 */
class Charset
{
	const UTF_8 = 'utf-8';
	const UTF_7 = 'utf-7';
	const UTF_7_IMAP = 'utf7-imap';
	const WIN_1250 = 'windows-1250';
	const WIN_1251 = 'windows-1251';
	const WIN_1252 = 'windows-1252';
	const WIN_1253 = 'windows-1253';
	const WIN_1254 = 'windows-1254';
	const WIN_1255 = 'windows-1255';
	const WIN_1256 = 'windows-1256';
	const WIN_1257 = 'windows-1257';
	const WIN_1258 = 'windows-1258';
	const ISO_8859_1 = 'iso-8859-1';
	const ISO_8859_8 = 'iso-8859-8';
	const ISO_8859_8_I = 'iso-8859-8-i';
	const ISO_2022_JP = 'iso-2022-jp';
	const CP858 = 'cp858';
}
