.attachmentItem {

	position: relative;

	display: inline-block;
	margin: 5px;
	max-width: 200px;
	min-width: 60px;
	overflow: hidden;
	list-style: none;
	line-height: 24px;
	border: 0;
	background-color: #fff;

	box-shadow: 0 1px 4px #ccc;
	box-shadow: 0 1px 5px rgba(0, 0, 0, 0.2);
	box-shadow: 0 0 0 1px rgba(0, 0, 0, 0.04), 0 1px 5px rgba(0, 0, 0, 0.1);

	border-radius: 3px;

	&.waiting {
		opacity: 0.6;
	}

	&.checked {
		box-shadow: 0 1px 4px #00a;
		box-shadow: 0 1px 5px rgba(0, 0, 255, 0.3);
		box-shadow: 0 0 0 1px rgba(0, 0, 255, 0.1), 0 1px 5px rgba(0, 0, 255, 0.2);
	}

	.checkedParent {
		position: absolute;
		top: 0;
		right: 0;
		padding: 1px 4px 0px 5px;
		background: #FAFAFA;
		border: 0px solid #CCC;
	}

	.attachmentSize {
		font-size: 12px;
		color: #999;
	}

	&.error {
		.attachmentIcon, .attachmentSize, .attachmentName {
			color: red;
		}
	}

	.attachmentIconParent {

		position: relative;

		height: 56px;
		width: 60px;

		background: none;

		.iconPreview, .iconBG, .iconMain, .iconProgress {
			position: absolute;
			top: 0;
			left: 0;

			display: inline-block;
			width: 100%;
			height: 100%;
		}

		.iconProgress {
			background: #eee;
			width: 0%;
		}

		.iconBG {
			font-size: 18px;
			text-align: center;
			color: #999;
			font-weight: bold;
			line-height: 55px;
			text-shadow: 0 1px 0 #fff;
			text-shadow: 0 1px 0 rgba(255, 255, 255, 0.8);
		}

		.iconPreview {
			display: none;

			background: #555;
			background-image: none;
			background: rgba(0, 0, 0, .5) !important;

			.attachmentIcon {
				color: #fff;
				text-shadow: 0 1px 0 #000;
			}
		}
	}

	.attachmentNameParent {

		position: relative;

		margin-left: 60px;
		padding: 4px 4px 3px 6px;
		min-width: 90px;

		cursor: pointer;

		color: #333;
		background: #fafafa;
		border-left: 1px solid #ddd;
	}

	.attachmentIcon {
		margin: 6px 0 0 13px;
		font-size: 36px;
		width: 36px;
		height: 36px;
		color: #aaa;
	}

	.attachmentIcon  {
		&.icon-none {
			display: none;
		}
		&.icon-file-certificate {
			margin-left: 15px;
		}
	}

	.attachmentIconText {
		display: inline-block;
		font-size: 28px;
		width: 60px;
		height: 56px;
		color: #aaa;
		line-height: 56px;
		text-align: center;
		font-style: normal;
	}

/*
	.attachmentIconParent.hasPreview.isImage {
		.iconMain {
			display: none;
		}
	}
*/

	.attachmentIconParent.hasPreview:hover {
		.iconPreview {
			display: inline-block;
		}
		.iconMain {
			display: none;
		}
	}

	.attachmentIconParent.hasPreplay:hover {
		.iconPreview {
			display: inline-block;
		}
		.iconMain {
			display: none;
		}
	}

	.showPreview, .showPreplay {
		display: none;
		cursor: pointer;
	}

	.attachmentIconParent.hasPreview {
		.showPreview {
			display: inline;
		}

		.hidePreview {
			display: none;
		}
	}
	.attachmentIconParent.hasPreplay {
		.showPreplay {
			display: inline;
		}

		.hidePreview {
			display: none;
		}
	}
}
