<div class="b-admin-about">
	<div class="form-horizontal">
		<div class="legend" data-i18n="TAB_ABOUT/LEGEND_ABOUT"></div>
		<div class="row" style="min-width: 800px;">
			<div class="span4">
				<div class="rl-logo"></div>
				<div style="margin-left: 30px;">
					2018 &copy; <span data-i18n="TAB_ABOUT/LABEL_ALL_RIGHTS_RESERVED"></span>
					<br />
					<a class="g-ui-link" href="http://www.rainloop.net/" target="_blank" style="padding-left: 0">http://rainloop.net/</a>
				</div>
			</div>
			<div class="rl-desc" style="float: left">
				<h1 style="margin-bottom: 0;">RainLoop</h1>
				<h4 style="margin-top: 0;">
					<span data-bind="text: version"></span>
					<span data-bind="visible: 'stable' !== coreType()" style="padding-left: 3px; color: red; font-weight: normal">
						(<span data-bind="text: coreType"></span> channel)
					</span>
				</h4>
				<h4 style="color: #aaa; font-weight: normal;" data-i18n="TAB_ABOUT/LABEL_TAG_HINT"></h4>
				<h5 style="font-weight: normal; margin-top: 40px;" data-bind="visible: !community">
					<div data-bind="visible: 'error' === statusType()">
						<i class="icon-warning" style="color: red"></i>
						&nbsp;&nbsp;
						<span data-bind="text: errorDesc"></span>
					</div>
					<div data-bind="visible: 'available' === statusType()">
						<i class="icon-bolt" style="color: red"></i>
						&nbsp;&nbsp;
						<span data-bind="html: coreRemoteVersionHtmlDesc"></span>
						<span data-bind="visible: '' !== coreRemoteRelease()">
							(<span data-bind="text: coreRemoteRelease"></span>)
						</span>
						<br />
						<span data-bind="if: coreWarning">
							<br />
							<i class="icon-warning" style="color: red"></i>
							&nbsp;&nbsp;
							<b data-i18n="HINTS/WARNING"></b>
							&nbsp;&nbsp;
							<span data-i18n="TAB_ABOUT/HINT_READ_CHANGE_LOG"></span>
							<br />
						</span>
						<br />
						<span data-bind="visible: coreUpdatable() && coreAccess()">
							<a class="btn" data-bind="click: updateCoreData">
								<i class="icon-sync"></i>
								&nbsp;&nbsp;
								<span data-i18n="TAB_ABOUT/BUTTON_UPDATE"></span>
							</a>
							&nbsp;&nbsp;&nbsp;
						</span>
						<span data-bind="visible: coreAccess()">
							<a class="btn" href="http://www.rainloop.net/downloads/" target="_blank">
								<i class="icon-download"></i>
								&nbsp;&nbsp;
								<span data-i18n="TAB_ABOUT/BUTTON_DOWNLOAD"></span>
							</a>
							&nbsp;&nbsp;&nbsp;
							<a class="btn" href="http://www.rainloop.net/changelog/" target="_blank">
								<i class="icon-file-text"></i>
								&nbsp;&nbsp;
								<span data-i18n="TAB_ABOUT/BUTTON_CHANGELOG"></span>
							</a>
						</span>
					</div>
					<div data-bind="visible: 'up-to-date' === statusType()">
						<i class="icon-ok" style="color: green"></i>
						&nbsp;&nbsp;
						<span data-i18n="TAB_ABOUT/HINT_IS_UP_TO_DATE"></span>
					</div>
					<div data-bind="visible: 'updating' === statusType()">
						<i class="icon-spinner animated"></i>
						&nbsp;&nbsp;
						<span data-i18n="TAB_ABOUT/LABEL_UPDATING"></span>&hellip;
					</div>
					<div data-bind="visible: 'checking' === statusType()">
						<i class="icon-spinner animated"></i>
						&nbsp;&nbsp;
						<span data-i18n="TAB_ABOUT/LABEL_CHECKING"></span>&hellip;
					</div>
				</h5>
			</div>
		</div>
	</div>
</div>