<?php

namespace RainLoop\Providers\Filters\Enumerations;

class ActionType
{
	const NONE = 'None';
	const MOVE_TO = 'MoveTo';
	const DISCARD = 'Discard';
	const VACATION = 'Vacation';
	const REJECT = 'Reject';
	const FORWARD = 'Forward';
}
