
.rl-view-model {
	&.RL-Login, &.RL-LoginNew, &.RL-AdminLogin, &.RL-About {
		position: relative;
		height: 100%;
		z-index: 5;
	}
}

.mfp-container .mfp-figure {
	min-height: 300px;
	min-width: 300px;
}

.ui-resizable-e {
	right: -1px;
	width: 5px;
}

.pswp__error-msg {
	color: #ccc;
	a, a:hover {
		color: #fff;
	}
}

.pswp__img--placeholder {
	opacity: 0;
}

.opentip-container {

	z-index: 2001 !important;
	.ot-content {
		font-size: 13px;
	}

	&.style-rainloopErrorTip .ot-content {
		color: red;
	}

	&.ot-show-effect-none, &.ot-hide-effect-none {
		transition: none !important;
	}

	&.ot-show-effect-fade {

		transition: none;

		&.ot-hidden {
			opacity: 0;
		}

		&.ot-going-to-show {
			opacity: 0;
			transition: opacity 0.2s ease-in-out;
		}
		&.ot-showing {
			opacity: 1;
			transition: opacity 0.2s ease-in-out;
		}
		&.ot-visible {
			opacity: 1;
			transition: none;
		}
	}
}

svg-icon {
	display: none;
}

.svg-icon {
	border: 0;
	outline: 0;

	height: 16px;
	width: 16px;
	display: inline-block;
	fill: #333;

	&.svg-icon-archive {
		height: 14px;
		width: 14px;
	}
}

.lg-backdrop.in {
	opacity: 0.8;
}


.btn .btn-text {
	padding-left: 8px;
}

.active-scale-transition {
	transition: transform .2s;
	&:active {
		transform: scale(.95);
	}
}

html.glass {
	input:-webkit-autofill,
	input:-webkit-autofill:hover,
	input:-webkit-autofill:focus,
	input:-webkit-autofill:active {
		-webkit-transition-delay: 9999s;
		-webkit-transition: color 9999s ease-out, background-color 9999s ease-out;
	}
}
