<div class="b-settings-contacts">
	<div class="form-horizontal g-ui-user-select-none">
		<div class="legend">
			<span class="i18n" data-i18n="SETTINGS_CONTACTS/LEGEND_CONTACTS"></span>
		</div>
		<div class="control-group">
			<div class="controls">
				<div data-bind="component: {
					name: 'Checkbox',
					params: {
						label: 'SETTINGS_CONTACTS/LABEL_CONTACTS_AUTOSAVE',
						value: contactsAutosave
					}
				}"></div>
			</div>
		</div>
	</div>
	<br />
	<div class="form-horizontal" data-bind="visible: allowContactsSync">
		<div class="legend">
			<span class="i18n" data-i18n="SETTINGS_CONTACTS/LEGEND_CONTACTS_SYNC"></span>
		</div>
		<div class="control-group">
			<div class="controls">
				<div data-bind="component: {
					name: 'Checkbox',
					params: {
						label: 'SETTINGS_CONTACTS/LABEL_CONTACTS_SYNC_ENABLE',
						value: enableContactsSync
					}
				}"></div>
			</div>
		</div>
		<div class="control-group">
			<label class="control-label">
				<span class="i18n" data-i18n="SETTINGS_CONTACTS/LABEL_CONTACTS_SYNC_AB_URL"></span>
			</label>
			<div class="controls">
				<input type="text" class="input-xxlarge"
					autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"
					data-bind="value: contactsSyncUrl" placeholder="https://" />
			</div>
		</div>
		<div class="control-group">
			<label class="control-label">
				<span class="i18n" data-i18n="SETTINGS_CONTACTS/LABEL_CONTACTS_SYNC_USER"></span>
			</label>
			<div class="controls">
				<input type="text" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"
					data-bind="value: contactsSyncUser" />
			</div>
		</div>
		<div class="control-group">
			<label class="control-label">
				<span class="i18n" data-i18n="SETTINGS_CONTACTS/LABEL_CONTACTS_SYNC_PASSWORD"></span>
			</label>
			<div class="controls">
				<input type="password" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"
					data-bind="value: contactsSyncPass" />
			</div>
		</div>

	</div>
</div>
