<div class="popups">
	<div class="modal hide b-languages-content g-ui-user-select-none" data-bind="modal: modalVisibility">
		<div class="modal-header">
			<button type="button" class="close" data-bind="command: cancelCommand">&times;</button>
			<h3>
				<span class="i18n" data-i18n="POPUPS_LANGUAGES/TITLE_LANGUAGES"></span>
			</h3>
		</div>
		<div class="modal-body" style="min-height: 150px;">
			<div data-bind="foreach: languages">
				<label class="lang-item" data-tooltip-i18n="off" data-tooltip-join="right" data-bind="click: function () { $root.changeLanguage(key); }, css: {'selected': selected, 'user': user},
					tooltip: function () { return $root.languageTooltipName(key); }">
					<span class="flag-wrapper">
						<span data-bind="css: 'flag flag-' + key.toLowerCase()" style=""></span>
					</span>
					<span class="lang-name" data-bind="text: fullName"></span>
					<i class="icon-ok pull-right" style="color: green; margin-top: 3px;" data-bind="visible: selected"></i>
				</label>
			</div>
		</div>
	</div>
</div>