<div class="e-item b-folder-system-item">
	<a class="e-link" data-bind="droppable: function (oEvent, oUi) { $root.messagesDrop($data, oUi); }, css: { 'selected': selected, 'selectable': selectable, 'print-count': 0 < printableUnreadCount().length, 'anim-action-class': actionBlink, 'is-inbox': isInbox }">
		<span class="badge pull-right count" data-bind="text: printableUnreadCount"></span>
		&nbsp;
		<span class="focused-poiner"></span>
		<span class="name" data-bind="text: localName()"></span>
		<span class="inbox-star-icon">
			<i class="icon-star"></i>
			<i class="icon-star-empty"></i>
		</span>
	</a>
</div>