<div class="recaptcha-control-group" id="recaptcha-place" style="display: none">
	<div class="controls">
		<div id="recaptcha_image" style="border-radius: 3px"></div>
	</div>
	<div class="controls">
		<div class="input-append">

			<input class="i18n input-block-level inputLoginForm inputCAPTCHA" type="text" autocomplete="off"
				id="recaptcha_response_field" data-i18n-placeholder="PLUGIN/LABEL_ENTER_THE_WORDS_ABOVE"
				data-i18n="[placeholder]PLUGIN/LABEL_ENTER_THE_WORDS_ABOVE" />

			<span class="add-on">
				<i class="icon-repeat" onclick="Recaptcha.reload()" style="cursor: pointer"></i>
			</span>
			
		</div>
	</div>
</div>