<div class="control-group" data-bind="css: {'error': actionValue.error}" style="margin-bottom: 0">
	<div class="controls">
		<textarea class="span5 i18n" data-bind="value: actionValue" style="height: 100px;"
			data-i18n="[placeholder]POPUPS_FILTER/REJECT_MESSAGE_LABEL"></textarea>
	</div>
</div>