<div class="b-settings b-admin-right">
	<div class="b-toolbar g-ui-user-select-none">
		<div class="btn-toolbar">
			<h4 class="pull-left">
				<i class="icon-spinner animated" style="margin-top: 2px" data-bind="style: {'visibility': adminManLoadingVisibility }"></i>
				&nbsp;&nbsp;
				RainLoop
				<span data-bind="visible: capa && !community">(<span data-i18n="TOP_PANEL/LABEL_PREMIUM"></span>)</span>
				&mdash;
				<span data-i18n="TOP_PANEL/LABEL_ADMIN_PANEL"></span>
				&nbsp;&nbsp;
				<span data-bind="visible: !community">(<span data-bind="text: adminDomain"></span>)</span>
			</h4>
			<div class="btn-group pull-right">
				<a class="btn single btn-narrow" data-bind="click: logoutClick">
					<i class="icon-power"></i>
				</a>
			</div>
		</div>
	</div>
	<div class="b-content" data-bind="nano: true, scrollerShadows: true">
		<div class="content g-scrollbox" data-scroller-shadows-content>
			<div class="content-wrapper">
				<div id="rl-settings-subscreen" class="b-settings-content" style="position: relative; z-index: 2"></div>
			</div>
		</div>
	</div>
</div>