<div class="b-login--welcome-content">
	<i class="icon-google iconsize50" style="cursor: pointer; padding-right: 20px" data-bind="command: googleFastCommand, visible: googleFastLoginEnabled"></i>
	<i class="icon-mail iconsize50" style="cursor: pointer" data-bind="click: displayMainForm"></i>
	<br /><br />
</div>