
.b-login-content .loginFormWrapper {
	width: 90%;
	.wrapper {
		padding: 30px 30px 10px 30px;
	}
}

.thm-login {

	width: 100% !important;
	width: calc(~'100% - 60px') !important;
	max-width: 303px !important;

	box-shadow: 0 1px 4px #ccc !important;
	box-shadow: 0 1px 5px rgba(0,0,0,.2) !important;
	box-shadow: 0 0 0 1px rgba(0,0,0,.04),0 1px 5px rgba(0,0,0,.1) !important;
}