
.b-login-content {

	height: 100%;
	text-align: center;

	.loginFormWrapper {

		display: inline-block;
		vertical-align: middle;
		text-align: center;
		perspective: 500px;

		.descWrapper {

			margin-bottom: 10px;

			.desc {
				font-size: 18px;
				padding: 2px;
			}
		}

		.alertError {
			max-width: 450px;
		}

		.wrapper-parent {
		}

		.loginForm {
			background-color: #efefef;
			text-align: left;
			color: #333;
			margin: 0;
			float: none;
			width: 303px;
		}

		.loginWelcomeForm {
			display: none;
		}

		.welcome-on {
			display: block;
		}

		.welcome-off {
			display: none;
		}

/*		.control-label {
			font-size: 16px;
			line-height: 30px;
		}*/

		.controls, .control-group {
			margin-bottom: 25px;
		}

		.wrapper {
			padding: 40px 40px 20px 40px;
		}

		#recaptcha_image img {
			border: 1px solid #ccc;
			border-radius: 3px;
		}

		.controls {
			.inputLoginForm, .inputEmail, .inputLogin, .inputPassword, .inputAdditionalCode {
				font-size: 18px;
				height: 40px;
				line-height: 20px;
				padding-left: 12px;
				padding-right: 12px;
			}
		}

		.languageLabel {
			margin-top: 5px;
		}

		.signMeLabel .e-checkbox {
			margin-top: 5px;
		}

		.languageLabel {
			margin-right: 5px;
		}

		.input-append .add-on {
			position: relative;
			height: 30px;
			background: none;
			margin-left: -35px;
			z-index: 1000;
			border: 0px;
		}

		.input-append .add-on i {
			font-size: 17px;
			line-height: 29px;
			color: #999;

			&.login-submit-icon{
				font-size: 24px;
				padding-top: 2px;
				cursor: pointer;
			}
		}

		.controls.error .add-on i, .control-group.error .add-on i {
			color: #b94a48;
		}
	}

	.buttonLogin {
		margin: 0;
	}

	.alert {
		margin: 0 0 20px 0;
		text-align: left;
	}

	.loginAfter {
		display: inline-block;
		height: 90%;
		vertical-align: middle;
		width: 0px;
	}

	.flag-selector {
		margin-bottom: 0;
	}

	.social-buttons {
		margin-top: 5px;
	}

	.social-button, .language-button {
		padding: 5px;
		outline: none;
	}
}

@keyframes loginRotation {
	to   {transform: rotate(1turn);}
}

.submitting-pane {
	position: relative;
	&.submitting:before{
		content: '';
		position: absolute;
		top: 0;
		left: 0;
		right: 0;
		bottom: 0;
		background: transparent;
		background: rgba(0,0,0,.3);
		z-index: 1051;
		border-radius: 8px;
	}
	html.cssanimations &.submitting:after{
		content: '';
		position: absolute;
		width: 60px;
		height: 60px;
		top: 50%;
		left: 50%;
		margin-top: -30px;
		margin-left: -30px;

		border: 8px solid transparent;
		border-color: transparent;
		border-top-color: #fff;
		animation: loginRotation 1s infinite ease-in-out;

		border-radius: 50%;
		z-index: 1052;
	}
 }

.btn-submit-icon-wrp {
	border: none;
	background: none;
	display: inline-block;
	margin: 0;
	padding: 0;
	outline: none;
	cursor: pointer;
}