<!DOCTYPE html>
<html class="no-js rl-booted-trigger rl-started-trigger glass" dir="{{BaseDir}}">
	<head>
		<noscript>
			<meta http-equiv="refresh" content="0; URL=./?/NoScript" />
		</noscript>
		<!--[if lte IE 8]>
			<meta http-equiv="refresh" content="0; URL=./?/BadBrowser" />
		<![endif]-->
		<meta charset="utf-8" />
		<meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1" />
		<meta name="viewport" content="{{BaseViewport}}">
		<meta name="apple-mobile-web-app-capable" content="yes" />
		<meta name="google" content="notranslate" />
		<meta name="robots" content="noindex,nofollow,noodp" />
		<meta name="AppBootData" content='{{RainloopBootData}}' id="app-boot-data" />
		{{BaseContentSecurityPolicy}}
		<title></title>
		{{BaseAppFaviconPngLinkTag}}
		{{BaseAppFaviconTouchLinkTag}}
		<link type="text/css" rel="stylesheet" href="{{BaseAppMainCssLink}}" />
		<link type="text/css" rel="stylesheet" href="{{BaseAppThemeCssLink}}" id="app-theme-link" />
		<link rel="manifest" href="{{BaseAppManifestLink}}" />
	</head>
	<body>
		<div id="rl-app"></div>
		<div id="rl-check"></div>
		<script type="text/javascript" data-cfasync="false" src="{{BaseAppBootScriptLink}}"></script>
	</body>
</html>
