.flag-wrapper {
	width: 24px;
	height: 16px;
	display: inline-block;
}

.flag {
	width: 16px;
	height: 11px;
	display: inline-block;
	background: url(flags.png) no-repeat;
}

.flag.flag-en {background-position: -144px -154px}
.flag.flag-en_us {background-position: -144px -154px}
.flag.flag-en_gb {background-position: -176px -44px}
.flag.flag-en_uk {background-position: -176px -44px}
.flag.flag-en_ca {background-position: -48px -22px}
.flag.flag-el_gr {background-position: -112px -55px}
.flag.flag-uk_ua {background-position: -96px -154px}
.flag.flag-ru_ru {background-position: -224px -121px}
.flag.flag-pt_br {background-position: -192px -11px}
.flag.flag-pl_pl {background-position: -32px -121px}
.flag.flag-nl_nl {background-position: -80px -110px}
.flag.flag-nb_no {background-position: -96px -110px}
.flag.flag-it_it {background-position: -208px -66px}
.flag.flag-de_de {background-position: -80px -33px}
.flag.flag-ro_ro {background-position: -192px -121px}
.flag.flag-fi_fi {background-position: -64px -44px}
.flag.flag-fr_fr {background-position: -144px -44px}
.flag.flag-fr_ca {background-position: -144px -44px}
.flag.flag-pt_pt {background-position: -112px -121px}
.flag.flag-pt_br {background-position: -192px -11px}
.flag.flag-ar_sa {background-position: 0 -132px}
.flag.flag-bg_bg {background-position: -80px -11px}
.flag.flag-es_es {background-position: -16px -44px}
.flag.flag-es_la {background-position: -16px -44px}
.flag.flag-et_ee {background-position: -192px -33px}
.flag.flag-fa_ir {background-position: -176px -66px}
.flag.flag-lt_lt {background-position: -80px -88px}
.flag.flag-lv_lv {background-position: -112px -88px}
.flag.flag-is_is {background-position: -192px -66px}
.flag.flag-hu_hu {background-position: -32px -66px}
.flag.flag-ko_kr {background-position: -144px -77px}
.flag.flag-ja_jp {background-position: -16px -77px}
.flag.flag-sl_si {background-position: -128px -132px}
.flag.flag-sk_sk {background-position: -144px -132px}
.flag.flag-tr_tr {background-position: -16px -154px}
.flag.flag-sv_se {background-position: -80px -132px}
.flag.flag-cs_cz {background-position: -64px -33px}
.flag.flag-zh_cn {background-position: -208px -22px}
.flag.flag-zh_tw {background-position: -64px -154px}
.flag.flag-zh_hk {background-position: -208px -22px}
.flag.flag-da_dk {background-position: -112px -33px}
.flag.flag-id_id {background-position: -64px -66px}
