
.g-ui-user-select-none {
//	webkit-touch-callout: none;
//	user-select: none;
//	standard-user-select: none;
//	touch-callout: none;
}

.g-ui-user-select-allow {
	webkit-touch-callout: inherit;
	user-select: inherit;
	standard-user-select: inherit;
	touch-callout: inherit;
}

.g-ui-clearfix {
	.clearfix();
}

.g-ui-link {
	color: #336699;
	text-decoration: underline;
	cursor: pointer;
	padding: 2px;

	&:focus {
		outline: 1px;
		outline-style: dotted;
	}
}

.g-ui-min-height-300 {
	min-height: 300px;
}

.g-ui-100-proc-height {
	height: 100%;
}

.g-ui-absolute-reset {
	position: absolute;
	top: 0;
	right: 0;
	bottom: 0;
	left: 0;
	padding: 0;
	margin: 0;
	border: 0;
	z-index: 0;
}

.g-ui-menu {

	max-height: 400px;
	max-width: 300px;
	overflow-y: auto;
	overflow-x: hidden;

	.e-link {
		text-decoration: none;
		cursor: pointer;
	}

	.e-item.selected > .e-link {
		background-color: #eee !important;
	}

	.e-item > .e-link:focus, .e-item > .e-link:hover {
		background-color: #555;
		background-image: none;
		color: #fff;
	}

	.e-item.disabled > .e-link {
		cursor: not-allowed;
		background-color: #fff;
		background-image: none;
		color: grey;
	}

	.e-item.disabled [class^="icon-"] {
		color: grey;
	}
}

.g-ui-table {

	display: table;
	width: 100%;

	.e-row {
		display: table-row;
	}

	.e-cell {
		display: table-cell;
		vertical-align: top;
		text-align: left;
	}
}

.g-ui-height-100proc {
	height: 100%;
}

.g-ui-resizable-delimiter-highlight {
	border: none;
	border-right: 6px solid #aaa;
}

.e-pagenator {

	.e-page {

		display: inline-block;
		color: #999;
		text-decoration: none;
		font-size: 22px;
		padding: 3px;
		cursor: pointer;

		&:hover .e-page-number {
			color: #555;
		}

		&.current .e-page-number {
			font-size: 25px;
			color: #333;
			border-bottom: 2px solid #000;
		}
	}
}

html.rgba .g-ui-resizable-delimiter-highlight {
	border-right-color: rgba(0, 0, 0, 0.2);
}

.settings-saved-trigger {

	display: inline-block;
	line-height: 17px;
	font-size: 16px;

	.animated {
		color: green;
	}

	.success {
		color: green;
		transition: opacity 0.5s linear;
		opacity: 0;
	}

	.error {
		color: red;
		transition: opacity 0.5s linear;
		opacity: 0;
	}

	.visible {
		opacity: 1;
	}
}

.settings-saved-trigger-input {
	&.success {
		border-color: green !important;
		transition: border-color 0.5s linear;
	}

	&.error {
		border-color: red !important;
		transition: border-color 0.5s linear;
	}
}

.display-none {
	display: none;
}

.e-spinner {
	display: none;
}

.e-powered, .e-mobile-switcher {
	margin-top: 8px;
	color: #333;
	a {
		color: #333;
		text-decoration: underline;
		&:hover {
			color: #333;
		}
	}
}

.e-languages {

	margin-top: 8px;
	color: #333;

	.flag-name {

		color: #333;
		border-bottom: 1px dashed #333;
		cursor: pointer;
		padding: 2px 0px;

		&:focus {
			outline: 1px;
			outline-style: dotted;
		}
	}
}

html.cssanimations {

	.e-spinner {
		display: block;
		margin: 5px auto 0;
		width: 100px;
		text-align: center;
	}

	.e-spinner .e-bounce {
		width: 15px;
		height: 15px;
		background-color: #ddd;
		margin: 0 5px;
		.box-shadow(0px 0px 3px rgba(0, 0, 0, 0.3));

		border-radius: 100%;
		display: inline-block;
		animation: bouncedelay 1.4s infinite ease-in-out;
		/* Prevent first frame from flickering when animation starts */
		animation-fill-mode: both;
	}

	.e-spinner .bounce1 {
		animation-delay: -0.32s;
	}

	.e-spinner .bounce2 {
		animation-delay: -0.16s;
	}
}

@keyframes bouncedelay {
  0%, 80%, 100% { transform: scale(0.0); }
  40% { transform: scale(1.0); }
}

.command.command-disabled.hide-on-disabled-command {
	display:none;
}
