<div class="messageListItem e-single-line" data-bind="css: lineAsCss()">
	<div class="sidebarParent">
		&nbsp;
	</div>
	<div class="delimiter"></div>
	<div class="wrapper">
		<div class="dateParent actionHandle dragHandle">
			<span class="date" data-moment-format="SHORT" data-moment-format-title="FULL" data-bind="moment: dateTimeStampInUTC"></span>
		</div>
		<div class="threadsParent" data-bind="visible: 1 < threadsLen()">
			<span class="threads-len">
				<span class="threads-len-data">
					<span data-bind="text: threadsLen"></span>
					<i class="icon-right-mini"></i>
				</span>
			</span>
		</div>
		<div class="checkedParent">
			<i class="checkboxMessage" data-bind="css: checked() ? 'checkboxMessage icon-checkbox-checked' : 'checkboxMessage icon-checkbox-unchecked'"></i>
		</div>
		<div class="flagParent">
			<span class="flagOn">
				<i class="icon-star"></i>
			</span>
			<span class="flagOnHalf">
				<i class="icon-star-empty"></i>
			</span>
			<span class="flagOff">
				<i class="icon-star-empty"></i>
			</span>
		</div>
		<div class="senderParent actionHandle dragHandle">
			<span class="replyFlag"><i class="icon-reply"></i>&nbsp;</span>
			<span class="forwardFlag"><i class="icon-forward"></i>&nbsp;</span>
			<span class="sender" data-bind="text: senderEmailsString, attr: {'title': senderClearEmailsString}"></span>
			&nbsp;
		</div>
		<div class="attachmentParent actionHandle dragHandle">
			<span class="attachment">
				<i data-bind="css: attachmentIconClass"></i>
			</span>
		</div>
		<div class="subjectParent actionHandle dragHandle">
			<b class="importantMark">!</b>
			<span class="subject emptySubjectText" data-bind="text: $root.emptySubjectValue"></span>
			<span class="subject-prefix" data-bind="text: subjectPrefix"></span><span class="subject-suffix" data-bind="text: subjectSuffix"></span>
		</div>
		<div class="clearfix"></div>
	</div>
</div>