<div class="b-settings-general g-ui-user-select-none">
	<div class="form-horizontal long-label">
		<div class="legend">
			<span class="i18n" data-i18n="SETTINGS_CHANGE_PASSWORD/LEGEND_CHANGE_PASSWORD"></span>
		</div>
		<div class="row">
			<div class="span6">
				<div class="control-group" data-bind="css: {'error': currentPassword.error}">
					<label class="control-label">
						<span class="i18n" data-i18n="SETTINGS_CHANGE_PASSWORD/LABEL_CURRENT_PASSWORD"></span>
					</label>
					<div class="controls">
						<input type="password" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"
							data-bind="textInput: currentPassword" />
					</div>
				</div>
				<div class="control-group" data-bind="css: {'error': passwordMismatch}">
					<label class="control-label">
						<span class="i18n" data-i18n="SETTINGS_CHANGE_PASSWORD/LABEL_NEW_PASSWORD"></span>
					</label>
					<div class="controls">
						<input type="password" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"
							data-bind="textInput: newPassword" />
					</div>
				</div>
				<div class="control-group" data-bind="css: {'error': passwordMismatch}">
					<label class="control-label">
						<span class="i18n" data-i18n="SETTINGS_CHANGE_PASSWORD/LABEL_REPEAT_PASSWORD"></span>
					</label>
					<div class="controls">
						<input type="password" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"
							data-bind="textInput: newPassword2" />
					</div>
				</div>
				<div class="control-group">
					<div class="controls">
						<a class="btn" data-bind="command: saveNewPasswordCommand, css: { 'btn-success': passwordUpdateSuccess, 'btn-danger': passwordUpdateError }">
							<i data-bind="css: {'icon-spinner animated': changeProcess(), 'icon-key': !changeProcess(), 'icon-white': passwordUpdateSuccess() || passwordUpdateError() }"></i>
							&nbsp;&nbsp;
							<span class="i18n" data-i18n="SETTINGS_CHANGE_PASSWORD/BUTTON_UPDATE_PASSWORD"></span>
						</a>
					</div>
				</div>
			</div>
			<div class="span4 alert alert-error alert-null-left-margin" data-bind="visible: '' !== errorDescription(), text: errorDescription"></div>
		</div>
	</div>
</div>