<tr class="e-item">
	<td class="package-name-parent">
		<i class="package-img icon-bolt"></i>
		<span class="package-name" data-bind="text: name"></span>
		<span class="package-installed pull-right" data-bind="text: installed"></span>
		<div class="package-desc" data-bind="text: desc"></div>
	</td>
	<td class="package-version-parent">
		<span class="package-version" data-bind="text: version"></span>
	</td>
	<td class="package-release-parent">
		<span class="package-release" data-bind="text: release"></span>
	</td>
	<td class="package-actions-parent">
		<i class="icon-download e-action button-download" data-bind="visible: !loading() && canBeUpdated, click: function () { $root.installPackage($data); }"></i>
		&nbsp;
		<i class="icon-trash e-action button-delete" data-bind="visible: !loading() && canBeDeleted, click: function () { $root.deletePackage($data); }"></i>
		<i class="icon-spinner animated" data-bind="visible: loading"></i>
	</td>
</tr>
