.popups {
	.b-advanced-search-content {
		&.modal {
			width: 750px;
			.control-label {
				width: 100px;
			}
			.controls {
				margin-left: 110px;
			}
		}
		.modal-header {
			background-color: #fff;
		}
	}
}
