
.b-admin-general {

	.flag-selector {
		padding-top: 5px;
	}

	.flag-name {

		border-bottom: 1px dashed #555;
		cursor: pointer;
		padding: 2px 0px;

		&:focus {
			outline: 1px;
			outline-style: dotted;
		}
	}
}

