
body {
	background-color: transparent;
}

label {
	cursor: pointer;
}

label.inline, span.inline {
	display: inline-block;
}

.close-custom {
	.close;
}

button.close-custom {
	padding: 0;
	cursor: pointer;
	background: transparent;
	border: 0;
	-webkit-appearance: none;
}

.minimize-custom {
	border: 0px solid #333;
	border-bottom-width: 3px;
	display: inline-block;
	float: right;
	height: 20px;
	width: 16px;
	font-size: 20px;
	font-weight: bold;
	line-height: 20px;
	margin-right: 15px;
	cursor: pointer;
}

.legend {
	display: block;
	width: 100%;
	padding: 0;
	margin-bottom: @baseLineHeight;
	font-size: @baseFontSize * 1.5;
	line-height: @baseLineHeight * 2;
	color: @grayDark;
	border: 0;
	border-bottom: 1px solid #e5e5e5;

	small {
	  font-size: @baseLineHeight * .75;
	  color: @grayLight;
	};
}

.legend + .control-group {
   margin-top: @baseLineHeight;
  -webkit-margin-top-collapse: separate;
}

.input-append input, .input-append select, .input-append .uneditable-input {
	.border-radius(3px);
}

select {
	width: 223px;
}

.btn .svg-icon {
	vertical-align: middle;
}

.btn-small.btn-small-small {
	padding: 3px 9px;
	font-size: 11px;
	line-height: 11px;
}

.btn.btn-thin {
	padding: 4px 9px;
}

.btn.btn-thin-2 {
	padding: 4px 7px;
}

.btn.btn-nowrap {
	white-space: nowrap;
}

.btn.btn-ellipsis {
	text-overflow: ellipsis;
	overflow: hidden;
	white-space: nowrap;
}

.btn.btn-transparent {
	background: none !important;
	border-color: transparent !important;
	text-shadow: none !important;
	box-shadow: none !important;
	opacity: .6;

	[class^="icon-"] {
		width: 19px;
		height: 19px;
		line-height: 20px;
		font-size: 19px;
	}

	&:hover {
		opacity: 1;
	}
}

.btn-group.open .btn.btn-transparent.dropdown-toggle {
	color: #BD362F;
	.caret {
		border-top-color: #BD362F;
	}
	[class^="icon-"]:before {
		color: #BD362F;
	}
}

.btn.btn-narrow {
	padding-left: 12px;
	padding-right: 12px;
}

.btn-group.btn-group-custom-margin > .btn + .btn {
	margin-left: 0px;
}

.btn.btn-large {
	font-size: 17px;
}

.dropdown-menu {
	border-radius: @btnBorderRadius;
}

.btn-group + .btn-group {
	margin-left: 3px;
}

.btn {
	border-radius: @btnBorderRadius;
	background-image: none;
	padding-left: 13px;
	padding-right: 13px;

	&.disabled, &[disabled] {
		opacity: 0.8;
		box-shadow: inset 0 1px 0 rgba(255,255,255,.2), 0 1px 2px rgba(0,0,0,.05);
	}

	border-color: rgba(0, 0, 0, 0.25) rgba(0, 0, 0, 0.25) rgba(0, 0, 0, 0.25);

//	&:active {
//		.box-shadow(~"inset 0 2px 4px rgba(0,0,0,.3), 0 0 0 rgba(0,0,0,.1)");
//	}

	html.textshadow &{

		text-shadow: 0 1px 0 #fff;

		&.btn-danger, &.btn-success, &.btn-primary, &.btn-warning {
			text-shadow: 0 1px 0 rgba(0, 0, 0, 0.3);
		}
	}
}

.btn.btn-dark-disabled-border {
	&.disabled, &[disabled] {
		border-color: #aaa;
	}
}

//.btn-group.open {
//  .dropdown-toggle {
//     .box-shadow(~"inset 0 2px 4px rgba(0,0,0,.3), 0 0 0 rgba(0,0,0,.1)");
//  }
//}

.btn.btn-success {
    background-color: #84AB04;
}

.btn-toolbar {
	margin-top: 0;
	margin-bottom: 0;
}

.dropdown-menu {
  border-radius: 0;
  box-shadow: 0px 1px 2px rgba(0,0,0,0.2);
}

.tooltip {
	font-size: 14px;
    z-index: 2000 !important;
	overflow: hidden;
	text-overflow: ellipsis;

	&.in {
		opacity: 1;
	}

	.tooltip-inner {
		max-width: 380px;
		text-shadow: 0px 0px 5px rgba(0, 0, 0, 0.2);
		padding: 5px 10px;
		border-radius: 2px;
		overflow: hidden;
		text-overflow: ellipsis;
	}

	.tooltip-big {
		font-size: 18px;
		padding: 0 10px;
		white-space: nowrap;
	}
}

.popover {
    width: auto;
    min-width: 250px;
    max-width: 550px;

    .popover-content pre {
        border: none;
        background: rgba(0, 0, 0, 0);
        padding: 0;
        word-break: normal;
    }
}

.btn-group > .btn {
	border-radius: 0 !important;
}

.btn-group > .btn.single {
	border-radius: @btnBorderRadius !important;
}

.btn-group > .btn.first {
	border-top-left-radius: @btnBorderRadius !important;
	border-bottom-left-radius: @btnBorderRadius !important;
}

.btn-group > .btn.last {
	border-top-right-radius: @btnBorderRadius !important;
	border-bottom-right-radius: @btnBorderRadius !important;
}

.btn-group + .btn-group {
	margin-left: 0;
}

.btn-group {
	margin-right: 3px;
}
.btn-group.btn-group-last {
	margin-right: 0;
}

.dropdown.colored-toggle.open .btn.dropdown-toggle {
	color: #BD362F;

	.caret {
		border-top-color: #BD362F;
	}

	[class^="icon-"]:before {
		color: #BD362F;
	}
}

textarea, input[type="text"], input[type="password"], input[type="email"], input[type="search"] {

	border: @rlInputBorderSize solid @inputBorder;

	&:focus {
		background-color: #fff;
		border: @rlInputBorderSize solid darken(@inputBorder, 20%);
		box-shadow: none;
	}
}

input[type="text"], input[type="password"], input[type="email"], input[type="search"] {
	height: @baseLineHeight - (@rlInputBorderSize - 1) * 2;
	line-height: @baseLineHeight - (@rlInputBorderSize - 1) * 2;
}

select {
	border: @rlInputBorderSize solid @inputBorder;
}

.alert a {
	text-decoration: underline;
}

.alert.alert-null-left-margin {
	margin-left: 0;
}

.alert a {
	color: #c09853;
}

.alert.alert-info a {
	color: #3a87ad;
}

.alert.alert-error a {
	color: #b94a48;
}

.nav-tabs > li > a {
	color: #555;
}

.popover {
	z-index: 2000;
}

html.no-rgba .modal {
	border-width: 0px !important;
}

.modal-backdrop, .modal-backdrop.fade.in {
	.opacity(20);
	transform: none;
}

.popups {

	position: absolute;
	top: 0;
	bottom: 0;
	left: 0;
	right: 0;
	z-index: 1100;
	overflow: auto;
//	-webkit-overflow-scrolling: touch;

	.modal {
		position: static;
		z-index: 1101;
		margin: 5% auto;
		background-color: transparent;
		overflow: hidden;

		.modal-body {
			background-color: #fff;
			max-height: none;
		}

		box-shadow: 0 5px 80px rgba(0,0,0,0.3);
	}
}

.modal.fade {
	.transition(none);
	top: 0;
}

.modal.fade.in {
	top: 0;
}

.modal-backdrop {
	.g-ui-user-select-none();
}

.modal.loginContent .modal-body, .modal.loginAdminContent .modal-body {
	background-color: transparent !important;
}

.picker.modal-dialog-bg, .picker.picker-dialog-bg {
	z-index: 2000 !important;
}

.picker.modal-dialog, .picker.picker-dialog {
	z-index: 2001 !important;
}

.form-horizontal.long-label .control-group {
	.control-label {
		width: 160px;
	}
	.controls {
		margin-left: 180px;
	}
}

.control-label.remove-padding-top {
	padding-top: 0;
}

html.rl-mobile {

	.popups .modal {
		width: 100% !important;
		width: calc(~'100% - 20px') !important;
	}

	.b-settings-content {
		padding: 10px;
		padding-left: 20px;
	}

	.list-table {
		width: 100%;
		max-width: 100%;
	}

	.form-horizontal {
		.control-group {
			.control-label {
				text-align: left;
				float: none;
			}
			.controls {
				margin-left: 0;
			}
		}
	}
}
.close-input-wrp {
	display: inline-block;
	position: relative;

	input {
		padding-right: 15px;
	}

	.close {
		position: absolute;
		right: 0;
		top: 5px;
		margin: 0 7px;
		z-index: 100;
		vertical-align: middle;
		opacity: .4;
		&:hover {
			opacity: .6;
		}
	}
}