<div>
	<div class="form-horizontal">
		<div class="legend">
			<span class="i18n" data-i18n="SETTINGS_CUSTOM_ADMIN_CUSTOM_TAB_PLUGIN/LEGEND_CUSTOM"></span>
			&nbsp;&nbsp;&nbsp;
			<i class="icon-spinner animated" style="margin-top: 5px" data-bind="visible: loading"></i>
		</div>
		<div class="control-group">
			<label class="control-label">
				<span class="i18n" data-i18n="SETTINGS_CUSTOM_ADMIN_CUSTOM_TAB_PLUGIN/LABEL_PHP_VERSION"></span>
			</label>
			<div class="controls" style="padding-top: 5px">
				<b data-bind="text: php"></b>
			</div>
		</div>
	</div>
</div>