<table class="table table-hover table-bordered b-admin-packages-list-table" data-bind="i18nUpdate: f">
	<colgroup>
		<col />
		<col style="width: 100px" />
		<col style="width: 100px" />
		<col style="width: 80px" />
	</colgroup>
	<tbody data-bind="template: { name: 'AdminSettingsPackagesListItem', foreach: f }"></tbody>
</table>