
.b-admin-domains {
	.process-place {
		width: 600px;
		padding: 14px 0;
		text-align: center;
		visibility: hidden;
	}
}

.b-admin-domains-list-table {

	width: 600px;

	.e-item {

		.e-action {
			cursor: pointer;
		}

		.domain-name {
			display: inline-block;
			word-break: break-all;
			box-sizing: border-box;
		}

		.domain-alias {
			display: inline-block;
			box-sizing: border-box;
			color: #bbb;
			padding-left: 5px;
		}

		&.disabled {
			.domain-name, .domain-alias {
				color: #bbb;
			}
		}

		.button-delete {
			margin-right: 15px;
			visibility: hidden;
			opacity: 0;
		}

		.delete-access {
			&.button-delete {
				visibility: visible;
				margin-right: 0;
				opacity: 1;
			}
		}

		.delete-domain, .disabled-domain {
			cursor: pointer;
			opacity: 0.5;
		}

		&.disabled .disabled-domain {
			opacity: 0.5;
		}

		.delete-domain, .disabled-domain {
			&:hover {
				opacity: 1;
			}
		}
	}
}
