
// MAIN
@main-color: #333;
@main-background-color: #e3e3e3;
@main-background-image: none;	// "images/background.png"
@main-background-size: inherit;

// LOADING
@loading-color: #000;			// #ddd
@loading-text-shadow: none;		// 0px 1px 0px rgba(0, 0, 0, 0.5);

// LOGIN
@login-color: #555;
@login-background-color: #eee;
@login-rgba-background-color: false;	// rgba(0,0,0,0.7)
@login-box-shadow: none;		// 0px 2px 10px rgba(0,0,0,0.5)
@login-border: 1px solid #ccc;
@login-border-radius: 7px;
@login-gradient-start: none;	// #f4f4f4
@login-gradient-end: none;		// #dfdfdf
@powered-color: #333;
@languages-color: #333;

@spinner-background: rgba(0,0,0,.3);
@spinner-color: #fff;

// MENU
@dropdown-menu-color: #333;
@dropdown-menu-background-color: #fff;
@dropdown-menu-hover-background-color: #444;
@dropdown-menu-hover-color: #eee;
@dropdown-menu-disabled-color: #999;
@dropdown-menu-selected-background-color: #eee;

// FOLDERS
@folders-color: #333;
@folders-disabled-color: #666;
@folders-selected-color: #eee;
@folders-selected-background-color: #333;
@folders-selected-rgba-background-color: false;
@folders-focused-color: #eee;
@folders-focused-background-color: #333;
@folders-focused-rgba-background-color: false;
@folders-hover-color: #eee;
@folders-hover-background-color: #333;
@folders-hover-rgba-background-color: false;
@folders-drop-color: #eee;
@folders-drop-background-color: #333;
@folders-drop-rgba-background-color: false;

// SETTINGS
@settings-menu-color: #333;
@settings-menu-disabled-color: #666;
@settings-menu-selected-color: #eee;
@settings-menu-selected-background-color: #333;
@settings-menu-selected-rgba-background-color: false;
@settings-menu-hover-color: #eee;
@settings-menu-hover-background-color: #333;
@settings-menu-hover-rgba-background-color: false;

// MESSAGE LIST
@message-list-toolbar-background-color: #eee;
@message-list-toolbar-rgba-background-color: false;
@message-list-toolbar-gradient-start: none;		// #f4f4f4
@message-list-toolbar-gradient-end: none;		// #dfdfdf

// MESSAGE
@message-background-color: #fff;
@message-rgba-background-color: false;
