<?php

/*
 * This file is part of MailSo.
 *
 * (c) 2014 Usenko Timur
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

class Net_IDNA2_Exception extends Exception
{
}

class Net_IDNA2_Exception_Nameprep extends Net_IDNA2_Exception
{
}
