<div class="b-admin-general">
	<div class="row" data-bind="visible: weakPassword">
		<div class="alert alert-warning span8" style="margin-top: 10px;">
			<h4 data-i18n="TAB_GENERAL/ALERT_WARNING"></h4>
			<br />
			<div data-i18n="[html]TAB_GENERAL/HTML_ALERT_WEAK_PASSWORD"></div>
		</div>
	</div>
	<div class="row" data-bind="visible: dataFolderAccess">
		<div class="alert alert-error span8" style="margin-top: 10px;">
			<h4 data-i18n="TAB_GENERAL/ALERT_WARNING"></h4>
			<br />
			<span data-i18n="TAB_GENERAL/ALERT_DATA_ACCESS"></span>
			&nbsp;
			<a href="https://www.rainloop.net/docs/installation/#notice"
				target="_blank"><strong>https://www.rainloop.net/docs/installation</strong></a>
		</div>
	</div>
	<div class="form-horizontal">
		<div class="legend i18n i18n-animation" data-i18n="TAB_GENERAL/LEGEND_INTERFACE"></div>
		<div class="control-group" style="margin-bottom: 10px;">
			<label class="control-label" data-i18n="TAB_GENERAL/LABEL_LANGUAGE"></label>
			<div class="controls">
				<div class="flag-selector">
					<span class="flag-wrapper">
						<span data-bind="css: 'flag flag-' + language().toLowerCase()"></span>
					</span>
					<span class="flag-name" tabindex="0" data-bind="text: languageFullName, click: selectLanguage, onSpace: selectLanguage, onEnter: selectLanguage"></span>
					&nbsp;&nbsp;
					<div data-bind="component: {
						name: 'SaveTrigger',
						params: { value: languageTrigger }
					}"></div>
				</div>
			</div>
		</div>
		<div class="control-group">
			<label class="control-label" data-i18n="TAB_GENERAL/LABEL_LANGUAGE_ADMIN"></label>
			<div class="controls">
				<div class="flag-selector">
					<span class="flag-wrapper">
						<span data-bind="css: 'flag flag-' + languageAdmin().toLowerCase()"></span>
					</span>
					<span class="flag-name" tabindex="0" data-bind="text: languageAdminFullName, click: selectLanguageAdmin, onSpace: selectLanguageAdmin, onEnter: selectLanguageAdmin"></span>
					&nbsp;&nbsp;
					<div data-bind="component: {
						name: 'SaveTrigger',
						params: { value: languageAdminTrigger }
					}"></div>
				</div>
			</div>
		</div>
		<div class="control-group">
			<label class="control-label" data-i18n="TAB_GENERAL/LABEL_THEME"></label>
			<div class="controls">
				<div data-bind="component: {
					name: 'Select',
					params: {
						options: themesOptions,
						value: theme,
						trigger: themeTrigger,
						optionsText: 'optText',
						optionsValue: 'optValue',
						size: 2
					}
				}"></div>
			</div>
		</div>
		<div class="control-group">
			<div class="controls">
				<div data-bind="component: {
					name: 'Checkbox',
					params: {
						label: 'TAB_GENERAL/LABEL_ALLOW_LANGUAGES_ON_SETTINGS',
						value: allowLanguagesOnSettings
					}
				}"></div>
				<div data-bind="component: {
					name: 'Checkbox',
					params: {
						label: 'TAB_GENERAL/LABEL_ALLOW_THEMES_ON_SETTINGS',
						value: capaThemes
					}
				}"></div>
				<div data-bind="component: {
					name: 'Checkbox',
					params: {
						label: 'TAB_GENERAL/LABEL_ALLOW_BACKGROUND_ON_SETTINGS',
						value: capaUserBackground
					}
				}"></div>
				<div data-bind="component: {
					name: 'Checkbox',
					params: {
						label: 'TAB_GENERAL/LABEL_NEW_FOLDER_MOVE',
						value: newMoveToFolder
					}
				}"></div>
			</div>
		</div>
		<div class="control-group">
			<div class="controls">
				<div data-bind="component: {
					name: 'Checkbox',
					params: {
						label: 'TAB_GENERAL/LABEL_SHOW_THUMBNAILS',
						value: capaAttachmentThumbnails,
						inline: false
					}
				}"></div>
				<div data-bind="component: {
					name: 'Checkbox',
					params: {
						label: 'TAB_GENERAL/LABEL_ALLOW_GRAVATAR',
						value: capaGravatar,
						inline: true
					}
				}"></div>
				(<a class="g-ui-link" href="http://en.gravatar.com/" target="_blank">http://en.gravatar.com/</a>)
			</div>
		</div>
		<div class="legend i18n i18n-animation" data-i18n="TAB_GENERAL/LEGEND_MAIN"></div>
		<div class="control-group">
			<label class="control-label" data-i18n="TAB_GENERAL/LABEL_ATTACHMENT_SIZE_LIMIT"></label>
			<div class="controls">
				<div data-bind="component: {
					name: 'Input',
					params: {
						label: 'MB',
						value: mainAttachmentLimit,
						trigger: attachmentLimitTrigger,
						size: 1
					}
				}"></div>
				<br />
				<br />
				<span class="well well-small" data-bind="visible: '' !== uploadDataDesc">
					<b>PHP:</b>
					&nbsp;
					<span data-bind="text: uploadDataDesc"></span>
				</span>
				&nbsp;&nbsp;&nbsp;
				<a href="#" target="_blank" class="btn btn-thin" data-bind="link: phpInfoLink()">
					<i class="icon-info"></i>
				</a>
			</div>
		</div>
		<br />
		<div class="control-group">
			<div class="controls">
				<div data-bind="component: {
					name: 'Checkbox',
					params: {
						label: 'TAB_GENERAL/LABEL_ALLOW_ADDITIONAL_ACCOUNTS',
						value: capaAdditionalAccounts
					}
				}"></div>
				<div data-bind="component: {
					name: 'Checkbox',
					params: {
						label: 'TAB_GENERAL/LABEL_ALLOW_IDENTITIES',
						value: capaIdentities
					}
				}"></div>
			</div>
		</div>
<!--
		<div class="control-group">
			<div class="controls">
				<div data-bind="component: {
					name: 'Checkbox',
					params: {
						label: 'TAB_GENERAL/LABEL_ALLOW_TEMPLATES',
						value: capaTemplates
					}
				}"></div>
			</div>
		</div>
-->
	</div>
</div>