<div class="popups">
	<div class="modal hide b-welcom-page-content g-ui-user-select-none"
			data-bind="modal: modalVisibility" style="width: 600px">

		<div class="modal-body" style="height: 450px; padding: 0">
			<div data-bind="if: welcomePageURL" style="height: 100%; width: 100%; margin: 0; padding: 0; position: relative">
				<iframe src="javascript:1;" tabindex="-1" frameborder="0"
					style="border: none; width: 100%; height: 100%; position: absolute; top: 0px; left: 0px; right: 0px; bottom: 0px;"
					data-bind="attr: {'src': welcomePageURL}"></iframe>
			</div>
		</div>

		<div class="modal-footer">
			<button class="btn" data-bind="command: cancelCommand, hasFocus: closeFocused">
				<i class="icon-remove"></i>
				&nbsp;&nbsp;
				<span class="i18n" data-i18n="POPUPS_WELCOME_PAGE/BUTTON_CLOSE"></span>
			</button>
		</div>

	</div>
</div>