
.b-settings-open-pgp {

	html.rl-mobile &{
		margin-right: 15px;
	}

	.process-place {
		text-align: center;
		width: 600px;
		padding: 14px 0;
	}

	.list-table {

		width: 750px;

		td {
			padding: 4px 8px;
			line-height: 30px;
		}

		.open-pgp-key-img {

			margin-right: 10px;
			vertical-align: top;

			.svg-icon {
				width: 12px;
				height: 12px;
			}
		}

		.open-pgp-key-id, .open-pgp-key-user {
			display: inline-block;
			word-break: break-all;
			box-sizing: border-box;
			line-height: 22px;
			cursor: default;
		}

		.open-pgp-key-user-address:first-child {
			line-height: 30px;
			margin-bottom: -4px;
		}
	}

	.open-pgp-key-item {

		.button-delete {
			margin-right: 15px;
			margin-top: 5px;
			visibility: hidden;
			opacity: 0;
		}

		.delete-access {
			&.button-delete {
				visibility: visible;
				margin-right: 0;
				opacity: 1;
			}
		}

		.delete-open-pgp-key, .view-open-pgp-key {
			cursor: pointer;
			opacity: 0.7;

			&:hover {
				opacity: 0.9;
			}
		}
	}
}
