<div class="adminSecurity g-ui-user-select-none">
	<div class="legend" data-i18n="TAB_SECURITY/LEGEND_SECURITY"></div>
	<div class="form-horizontal">
		<div class="control-group">
			<div class="controls">
				<div data-bind="component: {
					name: 'Checkbox',
					params: {
						value: capaTwoFactorAuth,
						label: 'TAB_SECURITY/LABEL_ALLOW_TWO_STEP',
						inline: true
					}
				}"></div>
				&nbsp;&nbsp;&nbsp;&nbsp;
				<div data-bind="component: {
					name: 'Checkbox',
					params: {
						value: capaTwoFactorAuthForce,
						enable: capaTwoFactorAuth,
						label: 'TAB_SECURITY/LABEL_FORCE_TWO_STEP',
						inline: true
					}
				}"></div>
			</div>
		</div>
	</div>
<!--
	<div class="form-horizontal">
		<div class="control-group" data-bind="visible: isTwoFactorDropperShown()">
			<div class="controls">
				<div class="input-append">
					<input type="text" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"
						data-bind="textInput: twoFactorDropperUser, hasfocus: twoFactorDropperUser.focused" placeholder="Email"/>
					<button class="btn">
						<i class="icon-spinner animated" data-bind="visible: true"></i>
						<span data-bind="visible: false">OK</span>
					</button>
				</div>
			</div>
		</div>
		<div class="control-group" data-bind="visible: !isTwoFactorDropperShown()">
			<div class="controls">
				<span class="g-ui-link" data-bind="click: showTwoFactorDropper">Drop user keys</span>
			</div>
		</div>
	</div>
	<br />
-->
	<div class="form-horizontal">
		<div class="control-group">
			<div class="controls">
				<div data-bind="component: {
					name: 'Checkbox',
					params: {
						value: useLocalProxyForExternalImages,
						label: 'TAB_SECURITY/LABEL_USE_IMAGE_PROXY'
					}
				}"></div>
				<div data-bind="component: {
					name: 'Checkbox',
					params: {
						value: capaOpenPGP,
						label: 'TAB_SECURITY/LABEL_ALLOW_OPEN_PGP',
						inline: true
					}
				}"></div>
			</div>
		</div>
		<div class="control-group">
			<div class="controls">
				<a href="#" target="_blank" class="g-ui-link" data-bind="link: phpInfoLink()" data-i18n="TAB_SECURITY/LABEL_SHOW_PHP_INFO"></a>
			</div>
		</div>
	</div>
	<br />
	<div class="form-horizontal">
		<div class="legend" data-i18n="TAB_SECURITY/LEGEND_ADMIN_PANEL_ACCESS_CREDENTIALS"></div>
		<div class="control-group">
			<label class="control-label" data-i18n="TAB_SECURITY/LABEL_CURRENT_PASSWORD"></label>
			<div class="controls">
				<input type="password" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"
					data-bind="textInput: adminPassword" />
			</div>
		</div>
		<br />
		<div class="control-group" data-bind="css: {'error': adminLoginError}">
			<label class="control-label" data-i18n="TAB_SECURITY/LABEL_NEW_LOGIN"></label>
			<div class="controls">
				<input type="text" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"
					data-bind="textInput: adminLogin" />
			</div>
		</div>
		<div class="control-group" data-bind="css: {'error': adminPasswordNewError}">
			<label class="control-label" data-i18n="TAB_SECURITY/LABEL_NEW_PASSWORD"></label>
			<div class="controls">
				<input type="password" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"
					data-bind="textInput: adminPasswordNew" />
			</div>
		</div>
		<div class="control-group" data-bind="css: {'error': adminPasswordNewError}">
			<label class="control-label" data-i18n="TAB_SECURITY/LABEL_REPEAT_PASSWORD"></label>
			<div class="controls">
				<input type="password" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"
					data-bind="textInput: adminPasswordNew2" />
			</div>
		</div>
		<div class="control-group">
			<div class="controls">
				<a class="btn" data-bind="command: saveNewAdminPasswordCommand, css: { 'btn-success': adminPasswordUpdateSuccess, 'btn-danger': adminPasswordUpdateError }">
					<i class="icon-key" data-bind="css: {'icon-white': adminPasswordUpdateSuccess() || adminPasswordUpdateError() }"></i>
					&nbsp;&nbsp;
					<span data-i18n="TAB_SECURITY/BUTTON_UPDATE_PASSWORD"></span>
				</a>
			</div>
		</div>
	</div>
	<div class="form-horizontal">
		<div class="legend" data-i18n="TAB_SECURITY/LEGEND_SSL"></div>
		<div class="control-group">
			<div class="controls">
				<div data-bind="component: {
					name: 'Checkbox',
					params: {
						value: verifySslCertificate,
						label: 'TAB_SECURITY/LABEL_REQUIRE_VERIFICATION',
						inline: true
					}
				}"></div>
				&nbsp;&nbsp;
				<span style="color:red">(<span data-i18n="HINTS/UNSTABLE"></span>)</span>
				<br />
				<div data-bind="component: {
					name: 'Checkbox',
					params: {
						enable: verifySslCertificate,
						value: allowSelfSigned,
						label: 'TAB_SECURITY/LABEL_ALLOW_SELF_SIGNED'
					}
				}"></div>
			</div>
		</div>
	</div>
</div>