.b-domain-alias-content {
	&.modal {
		width: 330px;
	}
	.modal-header {
		background-color: #fff;
	}

	.error-desc {
		color: red;
	}
}

.b-domain-content {

	&.modal {
		width: 810px;
	}

	.modal-header {
		background-color: #fff;
	}

	.modal-body {
		position: relative;
		overflow: hidden;
		width: 1600px;
		height: 400px;
		left: 0;
	}

	&.domain-edit .modal-body {
		height: 340px;
	}

	&.domain-second-page .modal-body {
		left: -800px;
	}

	.domain-desc {
		color: #666;
		line-height: 20px;
		background-color: #f9f9f9;
		padding: 8px;
		border: 1px solid #eee;
		border-radius: 3px;
		margin: -5px 0;
		i {
			font-style: normal;
			color: red;
		}
	}

	.error-desc {
		color: red;
	}

	.testing-done {
		&.imap-header, &.sieve-header, &.smtp-header {
			color: green;
			font-weight: bold;
		}
	}

	.testing-error {
		&.imap-header, &.sieve-header, &.smtp-header {
			color: red;
		}
	}
}