<label class="e-component e-checkbox inline" data-bind="css: { 'disabled': disable() || !enable() }">
	<input type="checkbox" data-bind="checked: value, disable: disable() || !enable()" style="margin: 0 0 1px 0;" />
<!-- ko if: labeled -->
	&nbsp;&nbsp;
	<span class="sub-label i18n" data-bind="attr: {'data-i18n': label}, css: {'i18n-animation': labelAnimated}"></span>
<!-- /ko -->
</label>