<div class="b-settings b-settins-right">
	<div class="b-toolbar" style="padding-left: 0">
		<div class="btn-toolbar" style="margin-top: 2px;">
			<div class="btn-group" data-bind="visible: mobile && leftPanelDisabled()">
				<a class="btn btn-thin-2 single" data-bind="click: showLeft">
					<i class="icon-right-middle"></i>
				</a>
			</div>
			<div class="btn-group" data-bind="visible: mobile && !leftPanelDisabled()" style="margin-left: -1px">
				<a class="btn btn-thin-2 single" data-bind="click: hideLeft">
					<i class="icon-left-middle"></i>
				</a>
			</div>
			<a class="btn button-back" data-bind="click: backToMailBoxClick">
				<i class="icon-left"></i>
				&nbsp;&nbsp;
				<span class="i18n i18n-animation" data-i18n="SETTINGS_LABELS/BUTTON_BACK"></span>
			</a>
		</div>
	</div>
	<div class="b-content" data-bind="nano: true, scrollerShadows: true">
		<div class="content g-scrollbox" data-scroller-shadows-content>
			<div class="content-wrapper">
				<div id="rl-settings-subscreen" class="b-settings-content" style="position: relative; z-index: 2"></div>
			</div>
		</div>
	</div>
</div>