<div class="popups">
	<div class="modal hide b-account-add-content g-ui-user-select-none" data-bind="modal: modalVisibility">
		<div>
			<div class="modal-header">
				<button type="button" class="close" data-bind="command: cancelCommand">&times;</button>
				<h3>
					<span data-bind="visible: isNew" class="i18n" data-i18n="POPUPS_ADD_ACCOUNT/TITLE_ADD_ACCOUNT"></span>
					<span data-bind="visible: !isNew()" class="i18n" data-i18n="POPUPS_ADD_ACCOUNT/TITLE_UPDATE_ACCOUNT"></span>
				</h3>
			</div>
			<div class="modal-body">
				<div class="form-horizontal">
					<div class="alert" data-bind="visible: '' !== submitError()">
						<button type="button" class="close-custom" data-bind="click: function () { submitError('') }">&times;</button>
						<span data-bind="text: submitError"></span>
						<div data-bind="visible: submitErrorAdditional">
							<br />
							<span data-bind="text: submitErrorAdditional"></span>
						</div>
					</div>
					<br />
					<div class="control-group" data-bind="css: {'error': emailError}">
						<label class="i18n control-label" data-i18n="LOGIN/LABEL_EMAIL"></label>
						<div class="controls">
							<label style="margin-top: 5px;" data-bind="visible: !isNew()"><strong data-bind="text: email"></strong></label>
							<input type="email" class="inputEmail input-xlarge" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"
								data-bind="visible: isNew, textInput: email, onEnter: addAccountCommand, hasfocus: emailFocus" />
						</div>
					</div>
					<div class="control-group" data-bind="css: {'error': passwordError}">
						<label class="i18n control-label" data-i18n="LOGIN/LABEL_PASSWORD"></label>
						<div class="controls">
							<input type="password" class="inputPassword input-xlarge" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"
								data-bind="value: password, onEnter: addAccountCommand" />
						</div>
					</div>
				</div>
			</div>
			<div class="modal-footer">
				<a class="btn buttonAddAccount" data-bind="command: addAccountCommand">
					<i data-bind="visible: isNew, css: {'icon-user-add': !submitRequest(), 'icon-spinner animated': submitRequest()}"></i>
					<i data-bind="visible: !isNew(), css: {'icon-ok': !submitRequest(), 'icon-spinner animated': submitRequest()}"></i>
					&nbsp;&nbsp;
					<span data-bind="visible: isNew" class="i18n" data-i18n="POPUPS_ADD_ACCOUNT/BUTTON_ADD_ACCOUNT"></span>
					<span data-bind="visible: !isNew()" class="i18n" data-i18n="POPUPS_ADD_ACCOUNT/BUTTON_UPDATE_ACCOUNT"></span>
				</a>
			</div>
		</div>
	</div>
</div>
