<?php

namespace SabreForRainLoop\VObject;

/**
 * This class contains the version number for the VObject package
 *
 * @copyright Copyright (C) 2007-2013 fruux GmbH (https://fruux.com/).
 * @author Evert Pot (http://evertpot.com/)
 * @license http://code.google.com/p/sabredav/wiki/License Modified BSD License
 */
class Version {

    /**
     * Full version number
     */
    const VERSION = '3.1.3';

}
