.popups {
	.b-identity-content {

		&.modal {
			width: 750px;
		}

		.modal-header {
			background-color: #fff;
		}

		.modal-body {
			overflow: hidden;
		}

		.control-label {
			width: 100px;
		}

		.controls {
			margin-left: 120px;
		}

		.e-signature-place {
			height: 200px;
		}

		.textEmail {
			margin-top: 5px;
			font-weight: bold;
		}
	}
}
