<?php

namespace SabreForRainLoop\DAV\Exception;

/**
 * FileNotFound
 *
 * Deprecated: Warning, this class is deprecated and will be removed in a
 * future version of SabreDAV. Please use SabreForRainLoop\DAV\Exception\NotFound instead.
 *
 * @copyright Copyright (C) 2007-2013 fruux GmbH (https://fruux.com/).
 * @author Evert Pot (http://evertpot.com/)
 * @deprecated Use SabreForRainLoop\DAV\Exception\NotFound instead
 * @license http://code.google.com/p/sabredav/wiki/License Modified BSD License
 */
class FileNotFound extends NotFound {

}

