<!DOCTYPE html>
<html>
	<head>
		{{RefreshMeta}}
		<meta charset="utf-8" />
		<meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1" />
		<meta name="google" content="notranslate" />
		<meta name="robots" content="noindex,nofollow,noodp" />
		<link type="text/css" rel="stylesheet" href="{{Stylesheet}}" />
		<title></title>
	</head>
	<body>
		<div class="social-icon-wrp redirecting">
			<div class="social-icon">
				<i class="icon-{{Icon}}"></i>
			</div>
		</div>
		{{Script}}
	</body>
</html>
