.b-plugin-content {

	&.modal {
		width: 660px;

		.modal-body {
			overflow: auto;
		}
	}

	.modal-header {
		background-color: #fff;
	}

	.information {
		display: inline-block;
		background-color: #ddd;
		border-radius: 10px;
		cursor: pointer;
		height: 25px;
		width: 30px;
		text-align: center;
		padding-top: 5px;
	}

	textarea {
		width: 400px;
		height: 70px;
	}
}